# -*- coding: utf-8 -*-
"""
Indexed Generators
"""
# ****************************************************************************
#       Copyright (C) 2013 Travis Scrimshaw <tcscrims at gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************
from sage.structure.category_object import normalize_names


class IndexedGenerators(object):
    r"""nodetex
    Abstract base class for parents whose elements consist of generators
    indexed by an arbitrary set.

    Options controlling the printing of elements:

    - ``prefix`` -- string, prefix used for printing elements of this
      module (optional, default 'x').  With the default, a monomial
      indexed by 'a' would be printed as ``x['a']``.

    - ``latex_prefix`` -- string or ``None``, prefix used in the `\LaTeX`
      representation of elements (optional, default ``None``). If this is
      anything except the empty string, it prints the index as a
      subscript.  If this is ``None``, it uses the setting for ``prefix``,
      so if ``prefix`` is set to "B", then a monomial indexed by 'a'
      would be printed as ``B_{a}``.  If this is the empty string, then
      don't print monomials as subscripts: the monomial indexed by 'a'
      would be printed as ``a``, or as ``[a]`` if ``latex_bracket`` is
      ``True``.

    - ``names`` -- dict with strings as values or list of strings (optional):
      a mapping from the indices of the generators to strings giving the
      generators explicit names. This is used instead of the print options
      ``prefix`` and ``bracket`` when ``names`` is specified.

    - ``latex_names`` -- dict with strings as values or list of strings
      (optional): same as ``names`` except using the `\LaTeX` representation

    - ``bracket`` -- ``None``, bool, string, or list or tuple of
      strings (optional, default ``None``): if ``None``, use the value of the
      attribute ``self._repr_option_bracket``, which has default value
      ``True``.  (``self._repr_option_bracket`` is available for backwards
      compatibility.  Users should set ``bracket`` instead.  If
      ``bracket`` is set to anything except ``None``, it overrides
      the value of ``self._repr_option_bracket``.)  If ``False``, do not
      include brackets when printing elements: a monomial indexed by
      'a' would be printed as ``B'a'``, and a monomial indexed by
      (1,2,3) would be printed as ``B(1,2,3)``.  If True, use "[" and
      "]" as brackets.  If it is one of "[", "(", or "{", use it and
      its partner as brackets.  If it is any other string, use it as
      both brackets.  If it is a list or tuple of strings, use the
      first entry as the left bracket and the second entry as the
      right bracket.

    - ``latex_bracket`` -- bool, string, or list or tuple of strings
      (optional, default False): if ``False``, do not include brackets in
      the LaTeX representation of elements.  This option is only
      relevant if ``latex_prefix`` is the empty string; otherwise,
      brackets are not used regardless.  If ``True``, use "\left[" and
      "\right]" as brackets.  If this is one of "[", "(", "\\{", "|",
      or "||", use it and its partner, prepended with "\left" and
      "\right", as brackets.  If this is any other string, use it as
      both brackets.  If this is a list or tuple of strings, use the
      first entry as the left bracket and the second entry as the
      right bracket.

    - ``scalar_mult`` -- string to use for scalar multiplication in
      the print representation (optional, default "*")

    - ``latex_scalar_mult`` -- string or ``None`` (default: ``None``),
      string to use for scalar multiplication in the latex
      representation.  If None, use the empty string if ``scalar_mult``
      is set to "*", otherwise use the value of ``scalar_mult``.

    - ``tensor_symbol`` -- string or ``None`` (default: ``None``),
      string to use for tensor product in the print representation. If
      ``None``, use  ``sage.categories.tensor.symbol`` and
      ``sage.categories.tensor.unicode_symbol``.

    - ``sorting_key`` -- a key function (default: ``lambda x: x``),
      to use for sorting elements in the output of elements

    - ``sorting_reverse`` -- bool (default: ``False``), if ``True``
      sort elements in reverse order in the output of elements

    - ``string_quotes`` -- bool (default: ``True``), if ``True`` then
      display string indices with quotes

    - ``iterate_key`` -- bool (default: ``False``) iterate through
      the elements of the key and print the result as comma separated
      objects for string output

    .. NOTE::

        These print options may also be accessed and modified using the
        :meth:`print_options` method, after the parent has been defined.

    EXAMPLES:

    We demonstrate a variety of the input options::

        sage: from sage.structure.indexed_generators import IndexedGenerators
        sage: I = IndexedGenerators(ZZ, prefix='A')
        sage: I._repr_generator(2)
        'A[2]'
        sage: I._latex_generator(2)
        'A_{2}'

        sage: I = IndexedGenerators(ZZ, bracket='(')
        sage: I._repr_generator(2)
        'x(2)'
        sage: I._latex_generator(2)
        'x_{2}'

        sage: I = IndexedGenerators(ZZ, prefix="", latex_bracket='(')
        sage: I._repr_generator(2)
        '[2]'
        sage: I._latex_generator(2)
        \left( 2 \right)

        sage: I = IndexedGenerators(ZZ, bracket=['|', '>'])
        sage: I._repr_generator(2)
        'x|2>'
    """
    def __init__(self, indices, prefix="x", **kwds):
        """
        Initialize ``self``.

        EXAMPLES:

        This is a mixin class, so don't need pickling equality::

            sage: I = sage.structure.indexed_generators.IndexedGenerators(ZZ)
            sage: TestSuite(I).run(skip='_test_pickling')
        """
        self._indices = indices

        # printing options for elements (set when initializing self).
        # This includes self._repr_option_bracket (kept for backwards
        # compatibility, declared to be True by default, needs to be
        # overridden explicitly).
        self._print_options = {'prefix': prefix,
                               'names': None,
                               'bracket': None,
                               'latex_bracket': False,
                               'latex_prefix': None,
                               'latex_names': None,
                               'scalar_mult': "*",
                               'latex_scalar_mult': None,
                               'tensor_symbol': None,
                               'string_quotes': True,
                               'sorting_key': lambda x: x,
                               'sorting_reverse': False,
                               'iterate_key': False}
        # 'bracket': its default value here is None, meaning that
        # the value of self._repr_option_bracket is used; the default
        # value of that attribute is True -- see immediately before
        # the method _repr_generator.  If 'bracket' is any value
        # except None, then it overrides the value of
        # self._repr_option_bracket.  Future users might consider
        # using 'bracket' instead of _repr_option_bracket.
        self.print_options(**kwds)

    def indices(self):
        """
        Return the indices of ``self``.

        EXAMPLES::

            sage: F = CombinatorialFreeModule(QQ, ['a', 'b', 'c'])
            sage: F.indices()
            {'a', 'b', 'c'}
        """
        return self._indices

    def prefix(self):
        """
        Return the prefix used when displaying elements of self.

        EXAMPLES::

            sage: F = CombinatorialFreeModule(QQ, ['a', 'b', 'c'])
            sage: F.prefix()
            'B'

        ::

            sage: X = SchubertPolynomialRing(QQ)
            sage: X.prefix()
            'X'
        """
        return self._print_options['prefix']

    def print_options(self, **kwds):
        """
        Return the current print options, or set an option.

        INPUT: all of the input is optional; if present, it should be
        in the form of keyword pairs, such as
        ``latex_bracket='('``.  The allowable keywords are:

        - ``prefix``
        - ``latex_prefix``
        - ``names``
        - ``latex_names``
        - ``bracket``
        - ``latex_bracket``
        - ``scalar_mult``
        - ``latex_scalar_mult``
        - ``tensor_symbol``
        - ``string_quotes``
        - ``sorting_key``
        - ``sorting_reverse``
        - ``iterate_key``

        See the documentation for :class:`IndexedGenerators` for
        descriptions of the effects of setting each of these options.

        OUTPUT: if the user provides any input, set the appropriate
        option(s) and return nothing.  Otherwise, return the
        dictionary of settings for print and LaTeX representations.

        EXAMPLES::

            sage: F = CombinatorialFreeModule(ZZ, [1,2,3], prefix='x')
            sage: F.print_options()
            {...'prefix': 'x'...}
            sage: F.print_options(bracket='(')
            sage: F.print_options()
            {...'bracket': '('...}

        TESTS::

            sage: sorted(F.print_options().items())
            [('bracket', '('), ('iterate_key', False),
             ('latex_bracket', False), ('latex_names', None),
             ('latex_prefix', None), ('latex_scalar_mult', None),
             ('names', None), ('prefix', 'x'),
             ('scalar_mult', '*'),
             ('sorting_key', <function ...<lambda> at ...>),
             ('sorting_reverse', False), ('string_quotes', True),
             ('tensor_symbol', None)]
            sage: F.print_options(bracket='[') # reset
        """
        # don't just use kwds.get(...) because I want to distinguish
        # between an argument like "option=None" and the option not
        # being there altogether.
        if kwds:
            for option in kwds:
                if option in self._print_options:
                    self._print_options[option] = kwds[option]
                else:
                    raise ValueError('{} is not a valid print option.'.format(option))
            return
        return self._print_options

    _repr_option_bracket = True

    def _parse_names(self, m, use_latex):
        """
        Return the name corresponding to ``m`` if it exists,
        otherwise return ``None``.

        EXAMPLES::

            sage: F = CombinatorialFreeModule(ZZ, [1,2,3], names='a,b,c',
            ....:                             latex_names='x,y,z')
            sage: F._parse_names(1, False)
            'a'
            sage: F._parse_names(1, True)
            'x'

            sage: F.print_options(latex_names=None)
            sage: F._parse_names(1, True)
            'a'

            sage: F.print_options(latex_names={1:'x', 2:'y'}, names=None)
            sage: F._parse_names(1, False) is None
            True
            sage: F._parse_names(1, True)
            'x'
            sage: F._parse_names(3, True) is None
            True

            sage: F.print_options(names={1:'a', 3:'c'}, latex_names=None)
            sage: F._parse_names(1, False)
            'a'
            sage: F._parse_names(1, True)
            'a'
            sage: F._parse_names(2, False) is None
            True
            sage: F._parse_names(2, True) is None
            True
        """
        names = self._print_options.get('names', None)
        if use_latex:
            latex_names = self._print_options.get('latex_names', None)
            if latex_names is not None:
                names = latex_names

        if names is not None:
            if isinstance(names, dict):
                try:
                    return names[m]
                except KeyError:
                    return None
            else: # treat it like a list
                try:
                    i = self._indices.rank(m)
                except (AttributeError, TypeError, KeyError, ValueError):
                    return None
                if i >= len(names):
                    return None
                try:
                    return names[i]
                except (AttributeError, TypeError, KeyError, ValueError):
                    return None
        return None

    def _repr_generator(self, m):
        """
        Return a string representing the generator indexed by ``m``.

        The output can be customized by setting any of the following
        options when initializing the parent:

        - ``prefix``
        - ``bracket``
        - ``scalar_mult``
        - ``names``
        - ``iterate_key``

        Alternatively, one can use the :meth:`print_options` method
        to achieve the same effect.  To modify the bracket setting,
        one can also set ``self._repr_option_bracket`` as long as one
        has *not* set the ``bracket`` option: if the
        ``bracket`` option is anything but ``None``, it overrides
        the value of ``self._repr_option_bracket``.

        See the documentation for :class:`CombinatorialFreeModule` for
        details on the initialization options.

        EXAMPLES::

            sage: F = CombinatorialFreeModule(QQ, ['a', 'b', 'c'])
            sage: e = F.basis()
            sage: e['a'] + 2*e['b']    # indirect doctest
            B['a'] + 2*B['b']

            sage: F = CombinatorialFreeModule(QQ, ['a', 'b', 'c'], prefix="F")
            sage: e = F.basis()
            sage: e['a'] + 2*e['b']    # indirect doctest
            F['a'] + 2*F['b']
            sage: F.print_options(string_quotes=False)
            sage: e['a'] + 2*e['b']
            F[a] + 2*F[b]

            sage: F = CombinatorialFreeModule(QQ, ['aa', 'bb', 'cc'], prefix="F")
            sage: e = F.basis()
            sage: F.print_options(iterate_key=True)
            sage: e['aa'] + 2*e['bb']
            F['a', 'a'] + 2*F['b', 'b']
            sage: F.print_options(string_quotes=False)
            sage: e['aa'] + 2*e['bb']
            F[a, a] + 2*F[b, b]

            sage: QS3 = CombinatorialFreeModule(QQ, Permutations(3), prefix="")
            sage: original_print_options = QS3.print_options()
            sage: a = 2*QS3([1,2,3])+4*QS3([3,2,1])
            sage: a                      # indirect doctest
            2*[[1, 2, 3]] + 4*[[3, 2, 1]]

            sage: QS3.print_options(bracket = False)
            sage: a              # indirect doctest
            2*[1, 2, 3] + 4*[3, 2, 1]

            sage: QS3.print_options(prefix='')
            sage: a              # indirect doctest
            2*[1, 2, 3] + 4*[3, 2, 1]

            sage: QS3.print_options(bracket="|", scalar_mult=" *@* ")
            sage: a              # indirect doctest
            2 *@* |[1, 2, 3]| + 4 *@* |[3, 2, 1]|

            sage: QS3.print_options(bracket="|", scalar_mult="*", iterate_key=True)
            sage: a              # indirect doctest
            2*|1, 2, 3| + 4*|3, 2, 1|

            sage: QS3.print_options(**original_print_options) # reset

        TESTS::

            sage: F = CombinatorialFreeModule(QQ, [('a', 'b'), ('c','d')])
            sage: e = F.basis()
            sage: e[('a','b')] + 2*e[('c','d')]    # indirect doctest
            B[('a', 'b')] + 2*B[('c', 'd')]

            sage: F.<a,b,c> = CombinatorialFreeModule(QQ)
            sage: a + 2*b
            a + 2*b

            sage: F = CombinatorialFreeModule(QQ, ZZ)
            sage: e = F.basis()
            sage: 3*e[1] + 2*e[-2]
            2*B[-2] + 3*B[1]
            sage: F.print_options(iterate_key=True)
            sage: 3*e[1] + 2*e[-2]
            2*B[-2] + 3*B[1]
        """
        ret = self._parse_names(m, False)
        if ret is not None:
            return ret
        bracket = self._print_options.get('bracket', None)
        bracket_d = {"{": "}", "[": "]", "(": ")"}
        if bracket is None:
            bracket = self._repr_option_bracket
        if bracket is True:
            left = "["
            right = "]"
        elif bracket is False:
            left = ""
            right = ""
        elif isinstance(bracket, (tuple, list)):
            left = bracket[0]
            right = bracket[1]
        elif bracket in bracket_d:
            left = bracket
            right = bracket_d[bracket]
        else:
            left = bracket
            right = bracket
        iterate_key = self._print_options.get('iterate_key', False)
        quotes = self._print_options.get('string_quotes', True)
        if iterate_key:
            try:
                m = iter(m)
            except TypeError:
                pass  # not iterable, so fallback to normal behavior
            else:
                if not quotes:
                    return self.prefix() + left + (', '.join(str(val) for val in m)) + right
                return self.prefix() + left + (', '.join(repr(val) for val in m)) + right
        if not quotes and isinstance(m, str):
            return self.prefix() + left + m + right
        return self.prefix() + left + repr(m) + right # mind the (m), to accept a tuple for m

    def _ascii_art_generator(self, m):
        r"""
        Return an ascii art representing the generator indexed by ``m``.

        TESTS::

            sage: R = NonCommutativeSymmetricFunctions(QQ).R()
            sage: ascii_art(R[1,2,2,4])
            R
               ****
              **
             **
             *
            sage: Partitions.options(diagram_str="#", convention="french")
            sage: ascii_art(R[1,2,2,4])
            R
             #
             ##
              ##
               ####
            sage: Partitions.options._reset()

            sage: F.<a,b,c> = CombinatorialFreeModule(QQ)
            sage: ascii_art(a + 2*b)
            a + 2*b
        """
        from sage.typeset.ascii_art import AsciiArt, ascii_art
        ret = self._parse_names(m, False)
        if ret is not None:
            return ascii_art(ret)

        pref = AsciiArt([self.prefix()])
        r = pref * (AsciiArt([" " * len(pref)]) + ascii_art(m))
        r._baseline = r._h - 1
        return r

    def _unicode_art_generator(self, m):
        r"""
        Return an unicode art representing the generator indexed by ``m``.

        TESTS::

            sage: R = NonCommutativeSymmetricFunctions(QQ).R()
            sage: unicode_art(R[1,2,2,4])
            R
               ┌┬┬┬┐
              ┌┼┼┴┴┘
             ┌┼┼┘
             ├┼┘
             └┘
            sage: Partitions.options.convention="french"
            sage: unicode_art(R[1,2,2,4])
            R
             ┌┐
             ├┼┐
             └┼┼┐
              └┼┼┬┬┐
               └┴┴┴┘
            sage: Partitions.options._reset()

            sage: F.<a,b,c> = CombinatorialFreeModule(QQ)
            sage: unicode_art(a + 2*b)
            a + 2*b
        """
        from sage.typeset.unicode_art import UnicodeArt, unicode_art
        ret = self._parse_names(m, False)
        if ret is not None:
            return unicode_art(ret)

        pref = UnicodeArt([self.prefix()])
        r = pref * (UnicodeArt([" " * len(pref)]) + unicode_art(m))
        r._baseline = r._h - 1
        return r

    def _latex_generator(self, m):
        r"""
        Return a `\LaTeX` for the generator indexed by ``m``.

        The output can be customized by setting any of the following
        options when initializing the parent:

        - ``prefix``
        - ``latex_prefix``
        - ``latex_bracket``
        - ``names``
        - ``latex_names``

        (Alternatively, one can use the :meth:`print_options` method
        to achieve the same effect.)

        See the documentation for :class:`CombinatorialFreeModule` for
        details on the initialization options.

        EXAMPLES::

            sage: F = CombinatorialFreeModule(QQ, ['a', 'b', 'c'])
            sage: e = F.basis()
            sage: latex(e['a'] + 2*e['b'])    # indirect doctest
            B_{a} + 2 B_{b}

            sage: F = CombinatorialFreeModule(QQ, ['a', 'b', 'c'], prefix="C")
            sage: e = F.basis()
            sage: latex(e['a'] + 2*e['b'])    # indirect doctest
            C_{a} + 2 C_{b}

            sage: QS3 = CombinatorialFreeModule(QQ, Permutations(3), prefix="", scalar_mult="*")
            sage: original_print_options = QS3.print_options()
            sage: a = 2*QS3([1,2,3])+4*QS3([3,2,1])
            sage: latex(a)                     # indirect doctest
            2 [1, 2, 3] + 4 [3, 2, 1]
            sage: QS3.print_options(latex_bracket=True)
            sage: latex(a)                     # indirect doctest
            2 \left[ [1, 2, 3] \right] + 4 \left[ [3, 2, 1] \right]
            sage: QS3.print_options(latex_bracket="(")
            sage: latex(a)                     # indirect doctest
            2 \left( [1, 2, 3] \right) + 4 \left( [3, 2, 1] \right)
            sage: QS3.print_options(latex_bracket=('\\myleftbracket', '\\myrightbracket'))
            sage: latex(a)                     # indirect doctest
            2 \myleftbracket [1, 2, 3] \myrightbracket + 4 \myleftbracket [3, 2, 1] \myrightbracket
            sage: QS3.print_options(**original_print_options) # reset

        TESTS::

            sage: F = CombinatorialFreeModule(QQ, [('a', 'b'), (0,1,2)])
            sage: e = F.basis()
            sage: latex(e[('a','b')])    # indirect doctest
            B_{('a', 'b')}
            sage: latex(2*e[(0,1,2)])    # indirect doctest
            2 B_{\left(0, 1, 2\right)}
            sage: F = CombinatorialFreeModule(QQ, [('a', 'b'), (0,1,2)], prefix="")
            sage: e = F.basis()
            sage: latex(2*e[(0,1,2)])    # indirect doctest
            2 \left(0, 1, 2\right)

            sage: F.<a,b,c> = CombinatorialFreeModule(QQ, latex_names='x,y,z')
            sage: latex(a + 2*b)
            x + 2 y
        """
        from sage.misc.latex import latex

        ret = self._parse_names(m, True)
        if ret is not None:
            return ret

        s = latex(m)
        if s.find('\\text{\\textt') != -1:
            # m contains "non-LaTeXed" strings, use string representation
            s = str(m)

        # dictionary with left-right pairs of "brackets".  put pairs
        # in here accept \\left and \\right as prefixes.
        bracket_d = {"{": "\\}", "[": "]", "(": ")", "\\{": "\\}",
                     "|": "|", "||": "||"}
        bracket = self._print_options.get('latex_bracket', False)
        if bracket is True:
            left = "\\left["
            right = "\\right]"
        elif bracket is False:
            left = ""
            right = ""
        elif isinstance(bracket, (tuple, list)):
            left = bracket[0]
            right = bracket[1]
        elif bracket in bracket_d:
            left = bracket
            right = bracket_d[bracket]
            if left == "{":
                left = "\\{"
            left = "\\left" + left
            right = "\\right" + right
        else:
            left = bracket
            right = bracket
        prefix = self._print_options.get('latex_prefix')
        if prefix is None:
            prefix = self._print_options.get('prefix')
        if prefix == "":
            return left + s + right
        return "%s_{%s}" % (prefix, s)

def split_index_keywords(kwds):
    """
    Split the dictionary ``kwds`` into two dictionaries, one containing
    keywords for :class:`IndexedGenerators`, and the other is everything else.

    OUTPUT:

    The dictionary containing only they keywords
    for :class:`IndexedGenerators`. This modifies the dictionary ``kwds``.

    .. WARNING::

        This modifies the input dictionary ``kwds``.

    EXAMPLES::

        sage: from sage.structure.indexed_generators import split_index_keywords
        sage: d = {'string_quotes': False, 'bracket': None, 'base': QQ}
        sage: split_index_keywords(d)
        {'bracket': None, 'string_quotes': False}
        sage: d
        {'base': Rational Field}
    """
    ret = {}
    for option in ['prefix', 'latex_prefix', 'bracket', 'latex_bracket',
                   'scalar_mult', 'latex_scalar_mult', 'tensor_symbol',
                   'sorting_key', 'sorting_reverse',
                   'string_quotes']:
        try:
            ret[option] = kwds.pop(option)
        except KeyError:
            pass
    return ret


def parse_indices_names(names, index_set, prefix, kwds=None):
    """
    Parse the names, index set, and prefix input, along with setting
    default values for keyword arguments ``kwds``.

    OUTPUT:

    The triple ``(N, I, p)``:

    - ``N`` is the tuple of variable names,
    - ``I`` is the index set, and
    - ``p`` is the prefix.

    This modifies the dictionary ``kwds``.

    .. NOTE::

        When the indices, names, or prefix have not been given, it
        should be passed to this function as ``None``.

    .. NOTE::

        For handling default prefixes, if the result will be ``None`` if
        it is not processed in this function.

    EXAMPLES::

        sage: from sage.structure.indexed_generators import parse_indices_names
        sage: d = {}
        sage: parse_indices_names('x,y,z', ZZ, None, d)
        (('x', 'y', 'z'), Integer Ring, None)
        sage: d
        {}
        sage: d = {}
        sage: parse_indices_names('x,y,z', None, None, d)
        (('x', 'y', 'z'), {'x', 'y', 'z'}, '')
        sage: d
        {'string_quotes': False}
        sage: d = {}
        sage: parse_indices_names(None, ZZ, None, d)
        (None, Integer Ring, None)
        sage: d
        {}

    ::

        sage: d = {'string_quotes':True, 'bracket':'['}
        sage: parse_indices_names(['a','b','c'], ZZ, 'x', d)
        (('a', 'b', 'c'), Integer Ring, 'x')
        sage: d
        {'bracket': '[', 'string_quotes': True}
        sage: parse_indices_names('x,y,z', None, 'A', d)
        (('x', 'y', 'z'), {'x', 'y', 'z'}, 'A')
        sage: d
        {'bracket': '[', 'string_quotes': True}
    """
    if index_set is None:
        if names is None:
            raise ValueError("either the indices or names must be given")

        if prefix is None:
            prefix = ''
        if kwds is None:
            kwds = {}
        kwds.setdefault('string_quotes', False)

    names, index_set = standardize_names_index_set(names, index_set, -1)

    return (names, index_set, prefix)


def standardize_names_index_set(names=None, index_set=None, ngens=None):
    """
    Standardize the ``names`` and ``index_set`` inputs.

    INPUT:

    - ``names`` -- (optional) the variable names
    - ``index_set`` -- (optional) the index set
    - ``ngens`` -- (optional) the number of generators

    If ``ngens`` is a negative number, then this does not check that
    the number of variable names matches the size of the index set.

    OUTPUT:

    A pair ``(names_std, index_set_std)``, where ``names_std`` is either
    ``None`` or a tuple of strings, and where ``index_set_std`` is a finite
    enumerated set.
    The purpose of ``index_set_std`` is to index the generators of some object
    (e.g., the basis of a module); the strings in ``names_std``, when they
    exist, are used for printing these indices. The ``ngens``

    If ``names`` contains exactly one name ``X`` and ``ngens`` is greater than
    1, then ``names_std`` are ``Xi`` for ``i`` in ``range(ngens)``.

    TESTS::

        sage: from sage.structure.indexed_generators import standardize_names_index_set
        sage: standardize_names_index_set('x,y')
        (('x', 'y'), {'x', 'y'})
        sage: standardize_names_index_set(['x','y'])
        (('x', 'y'), {'x', 'y'})
        sage: standardize_names_index_set(['x','y'], ['a','b'])
        (('x', 'y'), {'a', 'b'})
        sage: standardize_names_index_set('x,y', ngens=2)
        (('x', 'y'), {'x', 'y'})
        sage: standardize_names_index_set(index_set=['a','b'], ngens=2)
        (None, {'a', 'b'})
        sage: standardize_names_index_set('x', ngens=3)
        (('x0', 'x1', 'x2'), {'x0', 'x1', 'x2'})

        sage: standardize_names_index_set()
        Traceback (most recent call last):
        ...
        ValueError: the index_set, names, or number of generators must be specified
        sage: standardize_names_index_set(['x'], ['a', 'b'])
        Traceback (most recent call last):
        ...
        IndexError: the number of names must equal the size of the indexing set
        sage: standardize_names_index_set('x,y', ['a'])
        Traceback (most recent call last):
        ...
        IndexError: the number of names must equal the size of the indexing set
        sage: standardize_names_index_set('x,y,z', ngens=2)
        Traceback (most recent call last):
        ...
        IndexError: the number of names must equal the number of generators
        sage: standardize_names_index_set(index_set=['a'], ngens=2)
        Traceback (most recent call last):
        ...
        IndexError: the size of the indexing set must equal the number of generators
    """
    if names is not None:
        if ngens is None or ngens < 0:
            names = normalize_names(-1, names)
        else:
            names = normalize_names(ngens, names)

    if index_set is None:
        if names is None:
            # If neither is specified, we make range(ngens) the index set
            if ngens is None:
                raise ValueError("the index_set, names, or number of"
                                 " generators must be specified")
            index_set = tuple(range(ngens))
        else:
            # If only the names are specified, then we make the indexing set
            #   be the names
            index_set = tuple(names)

    from sage.sets.finite_enumerated_set import FiniteEnumeratedSet
    if isinstance(index_set, dict): # dict of {name: index} -- not likely to be used
        if names is not None:
            raise ValueError("cannot give index_set as a dict and names")
        names = normalize_names(-1, tuple(index_set.keys()))
        index_set = FiniteEnumeratedSet([index_set[n] for n in names])
    elif isinstance(index_set, str):
        index_set = FiniteEnumeratedSet(list(index_set))
    elif isinstance(index_set, (tuple, list)):
        index_set = FiniteEnumeratedSet(index_set)

    if ngens is None or ngens >= 0:
        if names is not None:
            if len(names) != index_set.cardinality():
                raise IndexError("the number of names must equal"
                                 " the size of the indexing set")
            if ngens is not None and len(names) != ngens:
                raise IndexError("the number of names must equal the"
                                 " number of generators")
        elif ngens is not None and index_set.cardinality() != ngens:
            raise IndexError("the size of the indexing set must equal"
                             " the number of generators")

    return (names, index_set)




r"""
Coxeter Group Algebras
"""
import functools
from sage.misc.cachefunc import cached_method
from sage.categories.algebra_functor import AlgebrasCategory

class CoxeterGroupAlgebras(AlgebrasCategory):

    class ParentMethods:

        def demazure_lusztig_operator_on_basis(self, w, i, q1, q2,
                                               side="right"):
            r"""
            Return the result of applying the `i`-th Demazure Lusztig
            operator on ``w``.

            INPUT:

            - ``w`` -- an element of the Coxeter group
            - ``i`` -- an element of the index set
            - ``q1,q2`` -- two elements of the ground ring
            - ``bar`` -- a boolean (default ``False``)

            See :meth:`demazure_lusztig_operators` for details.

            EXAMPLES::

                sage: W = WeylGroup(["B",3])
                sage: W.element_class._repr_=lambda x: "".join(str(i) for i in x.reduced_word())
                sage: K = QQ['q1,q2']
                sage: q1, q2 = K.gens()
                sage: KW = W.algebra(K)
                sage: w = W.an_element()
                sage: KW.demazure_lusztig_operator_on_basis(w, 0, q1, q2)
                (-q2)*323123 + (q1+q2)*123
                sage: KW.demazure_lusztig_operator_on_basis(w, 1, q1, q2)
                q1*1231
                sage: KW.demazure_lusztig_operator_on_basis(w, 2, q1, q2)
                q1*1232
                sage: KW.demazure_lusztig_operator_on_basis(w, 3, q1, q2)
                (q1+q2)*123 + (-q2)*12

            At `q_1=1` and `q_2=0` we recover the action of the
            isobaric divided differences `\pi_i`::

                sage: KW.demazure_lusztig_operator_on_basis(w, 0, 1, 0)
                123
                sage: KW.demazure_lusztig_operator_on_basis(w, 1, 1, 0)
                1231
                sage: KW.demazure_lusztig_operator_on_basis(w, 2, 1, 0)
                1232
                sage: KW.demazure_lusztig_operator_on_basis(w, 3, 1, 0)
                123

            At `q_1=1` and `q_2=-1` we recover the action of the
            simple reflection `s_i`::

                sage: KW.demazure_lusztig_operator_on_basis(w, 0, 1, -1)
                323123
                sage: KW.demazure_lusztig_operator_on_basis(w, 1, 1, -1)
                1231
                sage: KW.demazure_lusztig_operator_on_basis(w, 2, 1, -1)
                1232
                sage: KW.demazure_lusztig_operator_on_basis(w, 3, 1, -1)
                12
            """
            return (q1+q2) * self.monomial(w.apply_simple_projection(i,side=side)) - self.term(w.apply_simple_reflection(i, side=side), q2)

        def demazure_lusztig_operators(self, q1, q2, side="right", affine=True):
            r"""
            Return the Demazure Lusztig operators acting on ``self``.

            INPUT:

            - ``q1,q2`` -- two elements of the ground ring `K`
            - ``side`` -- ``"left"`` or ``"right"`` (default: ``"right"``);
              which side to act upon
            - ``affine`` -- a boolean (default: ``True``)

            The Demazure-Lusztig operator `T_i` is the linear map
            `R \to R` obtained by interpolating between the
            simple projection `\pi_i` (see
            :meth:`CoxeterGroups.ElementMethods.simple_projection`)
            and the simple reflection `s_i` so that `T_i` has
            eigenvalues `q_1` and `q_2`:

            .. MATH::

                (q_1 + q_2) \pi_i - q_2 s_i.

            The Demazure-Lusztig operators give the usual
            representation of the operators `T_i` of the `q_1,q_2`
            Hecke algebra associated to the Coxeter group.

            For a finite Coxeter group, and if ``affine=True``, the
            Demazure-Lusztig operators `T_1,\dots,T_n` are completed
            by `T_0` to implement the level `0` action of the affine
            Hecke algebra.

            EXAMPLES::

                sage: W = WeylGroup(["B",3])
                sage: W.element_class._repr_=lambda x: "".join(str(i) for i in x.reduced_word())
                sage: K = QQ['q1,q2']
                sage: q1, q2 = K.gens()
                sage: KW = W.algebra(K)
                sage: T = KW.demazure_lusztig_operators(q1, q2, affine=True)
                sage: x = KW.monomial(W.an_element()); x
                123
                sage: T[0](x)
                (-q2)*323123 + (q1+q2)*123
                sage: T[1](x)
                q1*1231
                sage: T[2](x)
                q1*1232
                sage: T[3](x)
                (q1+q2)*123 + (-q2)*12

                sage: T._test_relations()

            .. NOTE::

                For a finite Weyl group `W`, the level 0 action of the
                affine Weyl group `\tilde W` only depends on the
                Coxeter diagram of the affinization, not its Dynkin
                diagram. Hence it is possible to explore all cases
                using only untwisted affinizations.
            """
            from sage.combinat.root_system.hecke_algebra_representation import HeckeAlgebraRepresentation
            W = self.basis().keys()
            cartan_type = W.cartan_type()
            if affine and cartan_type.is_finite():
                cartan_type = cartan_type.affine()
            T_on_basis = functools.partial(self.demazure_lusztig_operator_on_basis, q1=q1, q2=q2, side=side)
            return HeckeAlgebraRepresentation(self, T_on_basis, cartan_type, q1, q2)

        @cached_method
        def demazure_lusztig_eigenvectors(self, q1, q2):
            r"""
            Return the family of eigenvectors for the Cherednik operators.

            INPUT:

            - ``self`` -- a finite Coxeter group `W`
            - ``q1,q2`` -- two elements of the ground ring `K`

            The affine Hecke algebra `H_{q_1,q_2}(\tilde W)` acts on
            the group algebra of `W` through the Demazure-Lusztig
            operators `T_i`. Its Cherednik operators `Y^\lambda` can
            be simultaneously diagonalized as long as `q_1/q_2` is not
            a small root of unity [HST2008]_.

            This method returns the family of joint eigenvectors,
            indexed by `W`.

            .. SEEALSO::

                - :meth:`demazure_lusztig_operators`
                - :class:`sage.combinat.root_system.hecke_algebra_representation.CherednikOperatorsEigenvectors`

            EXAMPLES::

                sage: W = WeylGroup(["B",2])
                sage: W.element_class._repr_=lambda x: "".join(str(i) for i in x.reduced_word())
                sage: K = QQ['q1,q2'].fraction_field()
                sage: q1, q2 = K.gens()
                sage: KW = W.algebra(K)
                sage: E = KW.demazure_lusztig_eigenvectors(q1,q2)
                sage: E.keys()
                Weyl Group of type ['B', 2] (as a matrix group acting on the ambient space)
                sage: w = W.an_element()
                sage: E[w]
                (q2/(-q1+q2))*2121 + ((-q2)/(-q1+q2))*121 - 212 + 12
            """
            W = self.basis().keys()
            if not W.cartan_type().is_finite():
                raise ValueError("the Demazure-Lusztig eigenvectors are only defined for finite Coxeter groups")
            result = self.demazure_lusztig_operators(q1, q2, affine=True).Y_eigenvectors()
            w0 = W.long_element()
            result.affine_lift = w0._mul_
            result.affine_retract = w0._mul_
            return result




"""
Fixing pickle for nested classes

As of Python 2.7, names for nested classes are set by Python in a way which is
incompatible with the pickling of such classes (pickling by name)::

    sage: class A:
    ....:     class B:
    ....:         pass
    sage: A.B.__name__
    'B'

instead of more natural ``"A.B"``. Furthermore upon pickling *and* unpickling a
class with name ``"A.B"`` in a module ``mod``, the standard cPickle module
searches for ``"A.B"`` in ``mod.__dict__`` instead of looking up ``"A"`` and
then ``"B"`` in the result. See: https://groups.google.com/forum/#!topic/sage-devel/bHBV9KWAt64

This module provides two utilities to workaround this issue:

- :func:`nested_pickle` "fixes" recursively the name of the subclasses of a
  class and inserts their fullname ``"A.B"`` in ``mod.__dict__``

- :class:`NestedClassMetaclass` is a metaclass ensuring that
  :func:`nested_pickle` is called on a class upon creation.

See also :mod:`sage.misc.nested_class_test`.

.. NOTE::

    In Python 3, nested classes, like any class for that matter, have
    ``__qualname__`` and the standard pickle module uses it for pickling and
    unpickling. Thus the pickle module searches for ``"A.B"`` first by looking
    up ``"A"`` in ``mod``, and then ``"B"`` in the result. So there is no
    pickling problem for nested classes in Python 3, and the two utilities are
    not really necessary. However, :class:`NestedClassMetaclass` is used widely
    in Sage and affects behaviors of Sage objects in other respects than in
    pickling and unpickling. Hence we keep :class:`NestedClassMetaclass` even
    with Python 3, for now. This module will be removed when we eventually drop
    support for Python 2.

EXAMPLES::

    sage: from sage.misc.nested_class import A1, nested_pickle

    sage: A1.A2.A3.__name__
    'A3'
    sage: A1.A2.A3
    <class 'sage.misc.nested_class.A1.A2.A3'>

    sage: nested_pickle(A1)
    <class 'sage.misc.nested_class.A1'>

    sage: A1.A2
    <class 'sage.misc.nested_class.A1.A2'>

    sage: A1.A2.A3
    <class 'sage.misc.nested_class.A1.A2.A3'>
    sage: A1.A2.A3.__name__
    'A1.A2.A3'

    sage: sage.misc.nested_class.__dict__['A1.A2'] is A1.A2
    True
    sage: sage.misc.nested_class.__dict__['A1.A2.A3'] is A1.A2.A3
    True

All of this is not perfect. In the following scenario::

    sage: class A1:
    ....:     class A2:
    ....:         pass
    sage: class B1:
    ....:     A2 = A1.A2

    sage: nested_pickle(A1)
    <class '__main__.A1'>
    sage: nested_pickle(B1)
    <class '__main__.B1'>
    sage: A1.A2
    <class '__main__.A1.A2'>
    sage: B1.A2
    <class '__main__.A1.A2'>

The name for ``"A1.A2"`` could potentially be set to ``"B1.A2"``. But that will work anyway.
"""

import sys
cdef dict sys_modules = sys.modules


__all__ = ['modify_for_nested_pickle', 'nested_pickle',
           'NestedClassMetaclass', 'MainClass'
           # Comment out to silence Sphinx warning about nested classes.
           #, 'SubClass', 'CopiedClass', 'A1'
           ]

cpdef modify_for_nested_pickle(cls, str name_prefix, module, first_run=True):
    r"""
    Modify the subclasses of the given class to be picklable, by
    giving them a mangled name and putting the mangled name in the
    module namespace.

    INPUT:

    - ``cls`` -- the class to modify

    - ``name_prefix`` -- the prefix to prepend to the class name

    - ``module`` -- the module object to modify with the mangled name

    - ``first_run`` -- optional bool (default ``True``): whether or not
      this function is run for the first time on ``cls``

    NOTE:

    This function would usually not be directly called. It is internally used
    in :class:`NestedClassMetaclass`.

    EXAMPLES::

        sage: from sage.misc.nested_class import *
        sage: class A(object):
        ....:     class B(object):
        ....:         pass
        sage: module = sys.modules['__main__']
        sage: A.B.__name__
        'B'
        sage: getattr(module, 'A.B', 'Not found')
        'Not found'
        sage: modify_for_nested_pickle(A, 'A', module)
        sage: A.B.__name__
        'A.B'
        sage: getattr(module, 'A.B', 'Not found')
        <class '__main__.A.B'>

    Here we demonstrate the effect of the ``first_run`` argument::

        sage: modify_for_nested_pickle(A, 'X', module)
        sage: A.B.__name__  # nothing changed
        'A.B'
        sage: modify_for_nested_pickle(A, 'X', module, first_run=False)
        sage: A.B.__name__
        'X.A.B'

    Note that the class is now found in the module under both its old and
    its new name::

        sage: getattr(module, 'A.B', 'Not found')
        <class '__main__.A.B'>
        sage: getattr(module, 'X.A.B', 'Not found')
        <class '__main__.A.B'>

    TESTS:

    The following is a real life example, that was enabled by the internal
    use of the``first_run`` in :trac:`9107`::

        sage: cython_code = [
        ....:  "from sage.structure.unique_representation import UniqueRepresentation",
        ....:  "class A1(UniqueRepresentation):",
        ....:  "    class B1(UniqueRepresentation):",
        ....:  "        class C1: pass",
        ....:  "    class B2:",
        ....:  "        class C2: pass"]
        sage: import os
        sage: cython(os.linesep.join(cython_code))

    Before :trac:`9107`, the name of ``A1.B1.C1`` would have been wrong::

        sage: A1.B1.C1.__name__
        'A1.B1.C1'
        sage: A1.B2.C2.__name__
        'A1.B2.C2'
        sage: A_module = sys.modules[A1.__module__]
        sage: getattr(A_module, 'A1.B1.C1', 'Not found').__name__
        'A1.B1.C1'
        sage: getattr(A_module, 'A1.B2.C2', 'Not found').__name__
        'A1.B2.C2'

    """
    cdef str name, dotted_name
    cdef str mod_name = module.__name__
    cdef str cls_name = cls.__name__+'.'
    cdef str v_name
    if first_run:
        for (name, v) in cls.__dict__.items():
            if isinstance(v, NestedClassMetaclass):
                v_name = v.__name__
                if v_name == name and v.__module__ == mod_name and getattr(module, v_name, None) is not v:
                    # OK, probably this is a nested class.
                    dotted_name = name_prefix + '.' + v_name
                    setattr(module, dotted_name, v)
                    modify_for_nested_pickle(v, name_prefix, module, False)
                    v.__name__ = dotted_name
            elif isinstance(v, type):
                v_name = v.__name__
                if v_name == name and v.__module__ == mod_name and getattr(module, v_name, None) is not v:
                    # OK, probably this is a nested class.
                    dotted_name = name_prefix + '.' + v_name
                    setattr(module, dotted_name, v)
                    modify_for_nested_pickle(v, dotted_name, module)
                    v.__name__ = dotted_name
    else:
        for (name, v) in cls.__dict__.items():
            if isinstance(v, type):
                v_name = v.__name__
                if v_name == cls_name + name and v.__module__ == mod_name:
                    # OK, probably this is a nested class.
                    dotted_name = name_prefix + '.' + v_name
                    setattr(module, dotted_name, v)
                    modify_for_nested_pickle(v, name_prefix, module, False)
                    v.__name__ = dotted_name


def nested_pickle(cls):
    r"""
    This decorator takes a class that potentially contains nested classes.
    For each such nested class, its name is modified to a new illegal
    identifier, and that name is set in the module.  For example, if
    you have::

        sage: from sage.misc.nested_class import nested_pickle
        sage: module = sys.modules['__main__']
        sage: class A(object):
        ....:     class B:
        ....:         pass
        sage: nested_pickle(A)
        <class '__main__.A'>

    then the name of class ``"B"`` will be modified to ``"A.B"``, and the ``"A.B"``
    attribute of the module will be set to class ``"B"``::

        sage: A.B.__name__
        'A.B'
        sage: getattr(module, 'A.B', 'Not found')
        <class '__main__.A.B'>

    In Python 2.6, decorators work with classes; then ``@nested_pickle``
    should work as a decorator::

        sage: @nested_pickle    # todo: not implemented
        ....: class A2(object):
        ....:     class B:
        ....:         pass
        sage: A2.B.__name__    # todo: not implemented
        'A2.B'
        sage: getattr(module, 'A2.B', 'Not found')    # todo: not implemented
        <class __main__.A2.B at ...>

    EXAMPLES::

        sage: from sage.misc.nested_class import *
        sage: loads(dumps(MainClass.NestedClass())) # indirect doctest
        <sage.misc.nested_class.MainClass.NestedClass object at 0x...>
    """
    modify_for_nested_pickle(cls, cls.__name__, sys_modules[cls.__module__])
    return cls


cdef class NestedClassMetaclass(type):
    r"""
    A metaclass for nested pickling.

    Check that one can use a metaclass to ensure nested_pickle is called on any
    derived subclass::

        sage: from sage.misc.nested_class import NestedClassMetaclass
        sage: class ASuperClass(object, metaclass=NestedClassMetaclass):
        ....:     pass
        sage: class A3(ASuperClass):
        ....:     class B(object):
        ....:         pass
        sage: A3.B.__name__
        'A3.B'
        sage: getattr(sys.modules['__main__'], 'A3.B', 'Not found')
        <class '__main__.A3.B'>
    """
    def __init__(self, *args):
        r"""
        This invokes the nested_pickle on construction.

        EXAMPLES::

            sage: from sage.misc.nested_class import NestedClassMetaclass
            sage: class A(object, metaclass=NestedClassMetaclass):
            ....:     class B(object):
            ....:         pass
            sage: A.B
            <class '__main__.A.B'>
            sage: getattr(sys.modules['__main__'], 'A.B', 'Not found')
            <class '__main__.A.B'>
        """
        modify_for_nested_pickle(self, self.__name__, sys_modules[self.__module__])


class MainClass(object, metaclass=NestedClassMetaclass):
    r"""
    A simple class to test nested_pickle.

    EXAMPLES::

        sage: from sage.misc.nested_class import *
        sage: loads(dumps(MainClass()))
        <sage.misc.nested_class.MainClass object at 0x...>
    """

    class NestedClass(object):
        r"""
        EXAMPLES::

            sage: from sage.misc.nested_class import *
            sage: loads(dumps(MainClass.NestedClass()))
            <sage.misc.nested_class.MainClass.NestedClass object at 0x...>
        """

        class NestedSubClass(object):
            r"""
            EXAMPLES::

                sage: from sage.misc.nested_class import *
                sage: loads(dumps(MainClass.NestedClass.NestedSubClass()))
                <sage.misc.nested_class.MainClass.NestedClass.NestedSubClass object at 0x...>
                sage: getattr(sage.misc.nested_class, 'MainClass.NestedClass.NestedSubClass')
                <class 'sage.misc.nested_class.MainClass.NestedClass.NestedSubClass'>
                sage: MainClass.NestedClass.NestedSubClass.__name__
                'MainClass.NestedClass.NestedSubClass'
            """
            def dummy(self, x, *args, r=(1,2,3.4), **kwds):
                """
                A dummy method to demonstrate the embedding of
                method signature for nested classes.

                TESTS::

                    sage: from sage.misc.nested_class import MainClass
                    sage: print(MainClass.NestedClass.NestedSubClass.dummy.__doc__)
                    NestedSubClass.dummy(self, x, *args, r=(1, 2, 3.4), **kwds)
                    File: sage/misc/nested_class.pyx (starting at line ...)
                    <BLANKLINE>
                                    A dummy method to demonstrate the embedding of
                                    method signature for nested classes.
                    ...

                """
                pass

class SubClass(MainClass):
    r"""
    A simple class to test nested_pickle.

    EXAMPLES::

        sage: from sage.misc.nested_class import SubClass
        sage: loads(dumps(SubClass.NestedClass()))
        <sage.misc.nested_class.MainClass.NestedClass object at 0x...>
        sage: loads(dumps(SubClass()))
        <sage.misc.nested_class.SubClass object at 0x...>
    """
    pass

nested_pickle(SubClass)

def _provide_SubClass():
    return SubClass

class CopiedClass(object):
    r"""
    A simple class to test nested_pickle.

    EXAMPLES::

        sage: from sage.misc.nested_class import CopiedClass
        sage: loads(dumps(CopiedClass.NestedClass()))
        <sage.misc.nested_class.MainClass.NestedClass object at 0x...>
        sage: loads(dumps(CopiedClass.NestedSubClass()))
        <sage.misc.nested_class.MainClass.NestedClass.NestedSubClass object at 0x...>
    """
    NestedClass = MainClass.NestedClass
    NestedSubClass = MainClass.NestedClass.NestedSubClass
    SubClass = _provide_SubClass()

nested_pickle(CopiedClass)

# Further classes for recursive tests

class A1:
    class A2:
        class A3:
            pass




"""
Examples of commutative additive monoids
"""
#*****************************************************************************
#  Copyright (C) 2008-2009 Nicolas M. Thiery <nthiery at users.sf.net>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#******************************************************************************

from sage.misc.cachefunc import cached_method
from sage.structure.parent import Parent
from sage.categories.all import CommutativeAdditiveMonoids
from .commutative_additive_semigroups import FreeCommutativeAdditiveSemigroup

class FreeCommutativeAdditiveMonoid(FreeCommutativeAdditiveSemigroup):
    r"""
    An example of a commutative additive monoid: the free commutative monoid

    This class illustrates a minimal implementation of a commutative monoid.

    EXAMPLES::

        sage: S = CommutativeAdditiveMonoids().example(); S
        An example of a commutative monoid: the free commutative monoid generated by ('a', 'b', 'c', 'd')

        sage: S.category()
        Category of commutative additive monoids

    This is the free semigroup generated by::

        sage: S.additive_semigroup_generators()
        Family (a, b, c, d)

    with product rule given by $a \times b = a$ for all $a, b$::

        sage: (a,b,c,d) = S.additive_semigroup_generators()

    We conclude by running systematic tests on this commutative monoid::

        sage: TestSuite(S).run(verbose = True)
        running ._test_additive_associativity() . . . pass
        running ._test_an_element() . . . pass
        running ._test_cardinality() . . . pass
        running ._test_category() . . . pass
        running ._test_construction() . . . pass
        running ._test_elements() . . .
          Running the test suite of self.an_element()
          running ._test_category() . . . pass
          running ._test_eq() . . . pass
          running ._test_new() . . . pass
          running ._test_nonzero_equal() . . . pass
          running ._test_not_implemented_methods() . . . pass
          running ._test_pickling() . . . pass
          pass
        running ._test_elements_eq_reflexive() . . . pass
        running ._test_elements_eq_symmetric() . . . pass
        running ._test_elements_eq_transitive() . . . pass
        running ._test_elements_neq() . . . pass
        running ._test_eq() . . . pass
        running ._test_new() . . . pass
        running ._test_not_implemented_methods() . . . pass
        running ._test_pickling() . . . pass
        running ._test_some_elements() . . . pass
        running ._test_zero() . . . pass
    """

    def __init__(self, alphabet=('a','b','c','d')):
        r"""
        The free commutative monoid

        INPUT:

         - ``alphabet`` -- a tuple of strings: the generators of the monoid

        EXAMPLES::

            sage: M = CommutativeAdditiveMonoids().example(alphabet=('a','b','c')); M
            An example of a commutative monoid: the free commutative monoid generated by ('a', 'b', 'c')

        TESTS::

            sage: TestSuite(M).run()

        """
        self.alphabet = alphabet
        Parent.__init__(self, category = CommutativeAdditiveMonoids())

    def _repr_(self):
        r"""
        TESTS::

            sage: M = CommutativeAdditiveMonoids().example(alphabet=('a','b','c'))
            sage: M._repr_()
            "An example of a commutative monoid: the free commutative monoid generated by ('a', 'b', 'c')"

        """
        return "An example of a commutative monoid: the free commutative monoid generated by %s"%(self.alphabet,)

    @cached_method
    def zero(self):
        r"""
        Returns the zero of this additive monoid, as per :meth:`CommutativeAdditiveMonoids.ParentMethods.zero`.

        EXAMPLES::

            sage: M = CommutativeAdditiveMonoids().example(); M
            An example of a commutative monoid: the free commutative monoid generated by ('a', 'b', 'c', 'd')
            sage: M.zero()
            0
        """
        return self(())

    class Element(FreeCommutativeAdditiveSemigroup.Element):
        def __bool__(self):
            """
            Check if ``self`` is not the zero of the monoid

            EXAMPLES::

                sage: M = CommutativeAdditiveMonoids().example()
                sage: bool(M.zero())
                False
                sage: [bool(m) for m in M.additive_semigroup_generators()]
                [True, True, True, True]
            """
            return any(x for x in self.value.values())

        __nonzero__ = __bool__

Example = FreeCommutativeAdditiveMonoid



from .factorization import Factorization

from .sequence import Sequence, seq

from .unique_representation import UniqueRepresentation

from .sage_object import SageObject

from .element import (
    canonical_coercion,
    coercion_model,
    get_coercion_model,
    coercion_traceback,
    parent
    )

from .parent import Parent

from .parent_gens import localvars

from .proof import all as proof

from sage.misc.lazy_import import lazy_import
lazy_import('sage.structure.formal_sum', ['FormalSums', 'FormalSum'])
del lazy_import

from .mutability import Mutability

from .element_wrapper import ElementWrapper



r"""
Generic numerical approximation function
"""

#*****************************************************************************
#       Copyright (C) 2016 Jeroen Demeyer <jdemeyer@cage.ugent.be>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  http://www.gnu.org/licenses/
#*****************************************************************************

from sage.structure.parent cimport Parent
from sage.structure.element cimport parent
cdef Parent CDF
from sage.rings.all import RealField, ComplexField, CDF


def numerical_approx_generic(x, prec):
    """
    Generic implementation of ``numerical_approx`` using coercion or
    conversion to a real or complex field.

    EXAMPLES::

        sage: from sage.arith.numerical_approx import numerical_approx_generic
        sage: numerical_approx_generic(pi, 20)
        3.1416
        sage: numerical_approx_generic(int(42), 20)
        42.000
        sage: numerical_approx_generic(float(4.2), 20)
        4.2000
    """
    P = parent(x)

    cdef Parent RR = RealField(prec)
    cmap = RR.coerce_map_from(P)
    if cmap is not None:
        return cmap(x)

    cdef Parent CC = ComplexField(prec)
    cmap = CC.coerce_map_from(P)
    if cmap is not None:
        return cmap(x)

    # Coercion didn't work: there are 3 possibilities:
    # (1) There is a coercion possible to a lower precision
    # (2) There is a conversion but no coercion
    # (3) The type doesn't convert at all

    # Figure out input precision to check for case (1)
    try:
        inprec = x.prec()
    except AttributeError:
        if prec > 53 and CDF.has_coerce_map_from(P):
            # If we can coerce to CDF, assume input precision was 53 bits
            inprec = 53
        else:
            # Otherwise, assume precision wasn't the issue
            inprec = prec

    if prec > inprec:
        raise TypeError("cannot approximate to a precision of %s bits, use at most %s bits" % (prec, inprec))

    # The issue is not precision, try conversion instead
    try:
        return RR(x)
    except (TypeError, ValueError):
        return CC(x)



r"""
Objects
"""
#*****************************************************************************
#  Copyright (C) 2005      David Kohel <kohel@maths.usyd.edu>
#                          William Stein <wstein@math.ucsd.edu>
#                2008      Teresa Gomez-Diaz (CNRS) <Teresa.Gomez-Diaz@univ-mlv.fr>
#                2008-2013 Nicolas M. Thiery <nthiery at users.sf.net>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  https://www.gnu.org/licenses/
#******************************************************************************

from sage.misc.cachefunc import cached_method
from sage.categories.category_singleton import Category_singleton
from sage.categories.homsets import HomsetsCategory

#############################################################
# Generic category (default when requesting category of
# an object using misc.functional.category
#############################################################

class Objects(Category_singleton):
    """
    The category of all objects
    the basic category

    EXAMPLES::

        sage: Objects()
        Category of objects
        sage: Objects().super_categories()
        []

    TESTS::

        sage: TestSuite(Objects()).run()
    """

    def additional_structure(self):
        """
        Return ``None``

        Indeed, by convention, the category of objects defines no
        additional structure.

        .. SEEALSO:: :meth:`Category.additional_structure`

        EXAMPLES::

            sage: Objects().additional_structure()
        """
        return None

    def super_categories(self):
        """
        EXAMPLES::

            sage: Objects().super_categories()
            []
        """
        return []

    def __contains__(self, x):
        """
        Anything is in the category of objects.

        EXAMPLES::

            sage: int(1) in Objects()
            True
            sage: ZZ     in Objects()
            True
            sage: 2/3    in Objects()
            True
        """
        return True

    class SubcategoryMethods:
        @cached_method
        def Homsets(self):
            r"""
            Return the category of homsets between objects of this category.

            EXAMPLES::

                sage: Sets().Homsets()
                Category of homsets of sets

                sage: Rings().Homsets()
                Category of homsets of unital magmas and additive unital additive magmas

            .. NOTE:: Background

                Information, code, documentation, and tests about the
                category of homsets of a category ``Cs`` should go in
                the nested class ``Cs.Homsets``. They will then be
                made available to homsets of any subcategory of
                ``Cs``.

                Assume, for example, that homsets of ``Cs`` are ``Cs``
                themselves. This information can be implemented in the
                method ``Cs.Homsets.extra_super_categories`` to make
                ``Cs.Homsets()`` a subcategory of ``Cs()``.

                Methods about the homsets themselves should go in the
                nested class ``Cs.Homsets.ParentMethods``.

                Methods about the morphisms can go in the nested class
                ``Cs.Homsets.ElementMethods``. However it's generally
                preferable to put them in the nested class
                ``Cs.MorphimMethods``; indeed they will then apply to
                morphisms of all subcategories of ``Cs``, and not only
                full subcategories.


            .. SEEALSO::

                :class:`~.covariant_functorial_construction.FunctorialConstruction`

            .. TODO::

                - Design a mechanism to specify that an axiom is
                  compatible with taking subsets. Examples:
                  ``Finite``, ``Associative``, ``Commutative`` (when
                  meaningful), but not ``Infinite`` nor ``Unital``.

                - Design a mechanism to specify that, when `B` is a
                  subcategory of `A`, a `B`-homset is a subset of the
                  corresponding `A` homset. And use it to recover all
                  the relevant axioms from homsets in super categories.

                - For instances of redundant code due to this missing
                  feature, see:

                  - :meth:`AdditiveMonoids.Homsets.extra_super_categories`
                  - :meth:`HomsetsCategory.extra_super_categories`
                    (slightly different nature)
                  - plus plenty of spots where this is not implemented.
            """
            return HomsetsCategory.category_of(self)

        @cached_method
        def Endsets(self):
            r"""
            Return the category of endsets between objects of this category.

            EXAMPLES::

                sage: Sets().Endsets()
                Category of endsets of sets

                sage: Rings().Endsets()
                Category of endsets of unital magmas and additive unital additive magmas

            .. SEEALSO::

                - :meth:`Homsets`
            """
            return self.Homsets()._with_axiom("Endset")

    class ParentMethods:
        """
        Methods for all category objects
        """



"Global proof preferences"

from sage.structure.sage_object import SageObject

class _ProofPref(SageObject):
    """
    An object that holds global proof preferences.  For now these are merely True/False flags for various parts of Sage that use probabilistic algorithms.
    A True flag means that the subsystem (such as linear algebra or number fields) should return results that are true unconditionally: the correctness should not depend on an algorithm with a nonzero probability of returning an incorrect answer or on the truth of any unproven conjectures.
    A False flag means that the subsystem can use faster methods to return answers that have a very small probability of being wrong.
    """
    def __init__(self, proof = True):
        self._require_proof = {}
        self._require_proof["arithmetic"] = proof
        self._require_proof["elliptic_curve"] = proof
        self._require_proof["linear_algebra"] = proof
        self._require_proof["number_field"] = proof
        self._require_proof["polynomial"] = proof
        self._require_proof["other"] = proof

    def arithmetic(self, t = None):
        """
        Controls the default proof strategy for integer arithmetic algorithms (such as primality testing).

        INPUT:

            t -- boolean or None

        OUTPUT:

            If t == True, requires integer arithmetic operations to (by default) return results that are true unconditionally: the correctness will not depend on an algorithm with a nonzero probability of returning an incorrect answer or on the truth of any unproven conjectures.
            If t == False, allows integer arithmetic operations to (by default) return results that may depend on unproven conjectures or on probabilistic algorithms.  Such algorithms often have a substantial speed improvement over those requiring proof.
            If t is None, returns the integer arithmetic proof status.

        EXAMPLES::

            sage: proof.arithmetic()
            True
            sage: proof.arithmetic(False)
            sage: proof.arithmetic()
            False
            sage: proof.arithmetic(True)
            sage: proof.arithmetic()
            True
        """
        if t is None:
            return self._require_proof["arithmetic"]
        self._require_proof["arithmetic"] = bool(t)

    def elliptic_curve(self, t = None):
        """
        Controls the default proof strategy for elliptic curve algorithms.

        INPUT:

            t -- boolean or None

        OUTPUT:

            If t == True, requires elliptic curve algorithms to (by default) return results that are true unconditionally: the correctness will not depend on an algorithm with a nonzero probability of returning an incorrect answer or on the truth of any unproven conjectures.
            If t == False, allows elliptic curve algorithms to (by default) return results that may depend on unproven conjectures or on probabilistic algorithms.  Such algorithms often have a substantial speed improvement over those requiring proof.
            If t is None, returns the current elliptic curve proof status.

        EXAMPLES::

            sage: proof.elliptic_curve()
            True
            sage: proof.elliptic_curve(False)
            sage: proof.elliptic_curve()
            False
            sage: proof.elliptic_curve(True)
            sage: proof.elliptic_curve()
            True
        """
        if t is None:
            return self._require_proof["elliptic_curve"]
        self._require_proof["elliptic_curve"] = bool(t)

    def linear_algebra(self, t = None):
        """
        Controls the default proof strategy for linear algebra algorithms.

        INPUT:

            t -- boolean or None

        OUTPUT:

            If t == True, requires linear algebra algorithms to (by default) return results that are true unconditionally: the correctness will not depend on an algorithm with a nonzero probability of returning an incorrect answer or on the truth of any unproven conjectures.
            If t == False, allows linear algebra algorithms to (by default) return results that may depend on unproven conjectures or on probabilistic algorithms.  Such algorithms often have a substantial speed improvement over those requiring proof.
            If t is None, returns the current linear algebra proof status.

        EXAMPLES::

            sage: proof.linear_algebra()
            True
            sage: proof.linear_algebra(False)
            sage: proof.linear_algebra()
            False
            sage: proof.linear_algebra(True)
            sage: proof.linear_algebra()
            True
        """
        if t is None:
            return self._require_proof["linear_algebra"]
        self._require_proof["linear_algebra"] = bool(t)

    def number_field(self, t = None):
        """
        Controls the default proof strategy for number field algorithms.

        INPUT:

            t -- boolean or None

        OUTPUT:

            If t == True, requires number field algorithms to (by default) return results that are true unconditionally: the correctness will not depend on an algorithm with a nonzero probability of returning an incorrect answer or on the truth of any unproven conjectures.
            If t == False, allows number field algorithms to (by default) return results that may depend on unproven conjectures or on probabilistic algorithms.  Such algorithms often have a substantial speed improvement over those requiring proof.
            If t is None, returns the current number field proof status.

        EXAMPLES::

            sage: proof.number_field()
            True
            sage: proof.number_field(False)
            sage: proof.number_field()
            False
            sage: proof.number_field(True)
            sage: proof.number_field()
            True
        """
        if t is None:
            return self._require_proof["number_field"]
        self._require_proof["number_field"] = bool(t)

    def polynomial(self, t = None):
        """
        Controls the default proof strategy for polynomial algorithms.

        INPUT:

            t -- boolean or None

        OUTPUT:

            If t == True, requires polynomial algorithms to (by default) return results that are true unconditionally: the correctness will not depend on an algorithm with a nonzero probability of returning an incorrect answer or on the truth of any unproven conjectures.
            If t == False, allows polynomial algorithms to (by default) return results that may depend on unproven conjectures or on probabilistic algorithms.  Such algorithms often have a substantial speed improvement over those requiring proof.
            If t is None, returns the current polynomial proof status.

        EXAMPLES::

            sage: proof.polynomial()
            True
            sage: proof.polynomial(False)
            sage: proof.polynomial()
            False
            sage: proof.polynomial(True)
            sage: proof.polynomial()
            True
        """
        if t is None:
            return self._require_proof["polynomial"]
        self._require_proof["polynomial"] = bool(t)


_proof_prefs = _ProofPref(True) #Creates the global object that stores proof preferences.

def get_flag(t = None, subsystem = None):
    """
    Used for easily determining the correct proof flag to use.

    EXAMPLES::

        sage: from sage.structure.proof.proof import get_flag
        sage: get_flag(False)
        False
        sage: get_flag(True)
        True
        sage: get_flag()
        True
        sage: proof.all(False)
        sage: get_flag()
        False
    """
    if t is None:
        if subsystem in ["arithmetic", "elliptic_curve", "linear_algebra", "number_field","polynomial"]:
            return _proof_prefs._require_proof[subsystem]
        else:
            return _proof_prefs._require_proof["other"]
    return t


class WithProof(object):
    """
    Use WithProof to temporarily set the value of one of the proof
    systems for a block of code, with a guarantee that it will be set
    back to how it was before after the block is done, even if there is an error.

    EXAMPLES::

        sage: proof.arithmetic(True)
        sage: with proof.WithProof('arithmetic',False):    # this would hang "forever" if attempted with proof=True
        ....:      print((10^1000 + 453).is_prime())
        ....:      print(1/0)
        Traceback (most recent call last):
        ...
        ZeroDivisionError: rational division by zero
        sage: proof.arithmetic()
        True
    """
    def __init__(self, subsystem, t):
        """
        TESTS::

            sage: proof.arithmetic(True)
            sage: P = proof.WithProof('arithmetic',False); P
            <sage.structure.proof.proof.WithProof object at ...>
            sage: P._subsystem
            'arithmetic'
            sage: P._t
            False
            sage: P._t_orig
            True
        """
        self._subsystem = str(subsystem)
        self._t = bool(t)
        self._t_orig = _proof_prefs._require_proof[subsystem]

    def __enter__(self):
        """
        TESTS::

            sage: proof.arithmetic(True)
            sage: P = proof.WithProof('arithmetic',False)
            sage: P.__enter__()
            sage: proof.arithmetic()
            False
            sage: proof.arithmetic(True)
        """
        _proof_prefs._require_proof[self._subsystem] = self._t

    def __exit__(self, *args):
        """
        TESTS::

            sage: proof.arithmetic(True)
            sage: P = proof.WithProof('arithmetic',False)
            sage: P.__enter__()
            sage: proof.arithmetic()
            False
            sage: P.__exit__()
            sage: proof.arithmetic()
            True
        """
        _proof_prefs._require_proof[self._subsystem] = self._t_orig




from sage.structure.element cimport Element
from .morphism cimport Morphism
from .map cimport Map
from .functor cimport Functor

cdef class Action(Functor):
    cdef readonly G
    cdef readonly op
    cdef readonly bint _is_left
    cdef US
    cdef underlying_set(self)

    cdef _act_convert(self, g, x)
    cpdef _act_(self, g, x)


cdef class InverseAction(Action):
    cdef Action _action
    cdef Map S_precomposition

cdef class PrecomposedAction(Action):
    cdef Action _action
    cdef Map G_precomposition
    cdef Map S_precomposition

cdef class ActionEndomorphism(Morphism):
    cdef Action _action
    cdef Element _g



# -*- coding: utf-8 -*-
"""
Element Wrapper

Wrapping Sage or Python objects as Sage elements.

AUTHORS:

- Nicolas Thiery (2008-2010): Initial version
- Travis Scrimshaw (2013-05-04): Cythonized version
"""

# ****************************************************************************
#       Copyright (C) 2008-2010 Nicolas M. Thiery <nthiery at users.sf.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from cpython.object cimport Py_EQ, Py_NE, Py_LE, Py_GE, PyObject_RichCompare

from sage.structure.coerce cimport coercion_model
from sage.structure.element cimport Element
from sage.structure.parent cimport Parent
from sage.structure.unique_representation import UniqueRepresentation
from copy import copy


cdef class ElementWrapper(Element):
    r"""
    A class for wrapping Sage or Python objects as Sage elements.

    EXAMPLES::

        sage: from sage.structure.element_wrapper import DummyParent
        sage: parent = DummyParent("A parent")
        sage: o = ElementWrapper(parent, "bla"); o
        'bla'
        sage: isinstance(o, sage.structure.element.Element)
        True
        sage: o.parent()
        A parent
        sage: o.value
        'bla'

    Note that ``o`` is not *an instance of* ``str``, but rather
    *contains a* ``str``. Therefore, ``o`` does not inherit the string
    methods. On the other hand, it is provided with reasonable default
    implementations for equality testing, hashing, etc.

    The typical use case of ``ElementWrapper`` is for trivially
    constructing new element classes from pre-existing Sage or Python
    classes, with a containment relation. Here we construct the
    tropical monoid of integers endowed with ``min`` as
    multiplication. There, it is desirable *not* to inherit the
    ``factor`` method from ``Integer``::

        sage: class MinMonoid(Parent):
        ....:     def _repr_(self):
        ....:         return "The min monoid"
        ....:
        sage: M = MinMonoid()
        sage: class MinMonoidElement(ElementWrapper):
        ....:     wrapped_class = Integer
        ....:
        ....:     def __mul__(self, other):
        ....:         return MinMonoidElement(self.parent(), min(self.value, other.value))
        sage: x = MinMonoidElement(M, 5); x
        5
        sage: x.parent()
        The min monoid
        sage: x.value
        5
        sage: y = MinMonoidElement(M, 3)
        sage: x * y
        3

    This example was voluntarily kept to a bare minimum. See the
    examples in the categories (e.g. ``Semigroups().example()``) for
    several full featured applications.

    .. WARNING::

        Versions before :trac:`14519` had parent as the second argument and
        the value as the first.
    """
    def __init__(self, parent, value):
        """
        EXAMPLES::

            sage: from sage.structure.element_wrapper import DummyParent
            sage: a = ElementWrapper(DummyParent("A parent"), 1)

        TESTS::

            sage: TestSuite(a).run(skip = "_test_category")

        .. NOTE::

            :class:`ElementWrapper` is not intended to be used directly,
            hence the failing category test.
        """
        Element.__init__(self, parent=parent)
        self.value = value

    def __getstate__(self):
        """
        Return a tuple describing the state of your object.

        This emulates :meth:`Element.__getstate__`, playing as if
        ``self.value`` was in the dictionary of ``self`` as it used to
        be before :trac:`14519`.

        EXAMPLES::

            sage: from sage.structure.element_wrapper import DummyParent
            sage: P = DummyParent("A parent")
            sage: a = ElementWrapper(P, 1)
            sage: a.__getstate__()
            (A parent, {'value': 1})
            sage: class A(ElementWrapper):
            ....:     pass
            sage: a = A(P, 1)
            sage: a.x = 2
            sage: a.__getstate__() == (P, {'value': 1, 'x': 2})
            True
        """
        try:
            d = self.__dict__
        except AttributeError:
            d = {}
        else:
            d = d.copy()
        d['value'] = self.value
        return (self._parent, d)

    def __setstate__(self, state):
        r"""
        Initialize the state of the object from data saved in a pickle.

        This emulates :meth:`Element.__setstate__`, playing as if
        ``self.value`` was to be put in the dictionary of ``self`` as
        it used to be before :trac:`14519`.

        EXAMPLES::

            sage: from sage.structure.element_wrapper import DummyParent
            sage: class A(ElementWrapper):
            ....:     pass
            sage: a = A(DummyParent("A parent"), 1)
            sage: a.__setstate__((DummyParent("Another parent"), {'value':0,'x':3}))
            sage: a.parent()
            Another parent
            sage: a.value
            0
            sage: a.x
            3

        TESTS::

            sage: a = A(DummyParent("A parent"), 1)
            sage: import __main__; __main__.A = A # Fake A being defined in a python module
            sage: a.x = 2
            sage: a == loads(dumps(a))
            True
            sage: a = ElementWrapper(DummyParent("A parent"), 1)
            sage: a == loads(dumps(a))
            True
        """
        # Make sure the first part of the state is the parent
        if not isinstance(state[0], Parent):
            state[0], state[1] = state[1], state[0]
        self._set_parent(state[0])
        d = state[1].copy()
        self.value = d.pop('value')
        if d:
            self.__dict__ = d

    def _repr_(self):
        """
        EXAMPLES::

            sage: from sage.structure.element_wrapper import DummyParent
            sage: ElementWrapper(DummyParent("A parent"), 1)
            1
        """
        return repr(self.value)

    def _latex_(self):
        r"""
        EXAMPLES::

            sage: from sage.structure.element_wrapper import DummyParent
            sage: ElementWrapper(DummyParent("A parent"), 1)._latex_()
            1
            sage: ElementWrapper(DummyParent("A parent"), 3/5)._latex_()
            \frac{3}{5}
        """
        from sage.misc.latex import latex
        return latex(self.value)

    def _ascii_art_(self):
        r"""
        EXAMPLES::

            sage: from sage.structure.element_wrapper import DummyParent
            sage: ElementWrapper(DummyParent("A parent"), 1)._ascii_art_()
            1
            sage: x = var('x')
            sage: ElementWrapper(DummyParent("A parent"), x^2 + x)._ascii_art_()
             2
            x  + x
        """
        from sage.typeset.ascii_art import ascii_art
        return ascii_art(self.value)

    def _unicode_art_(self):
        """
        Return a unicode art representation of ``self``.

        EXAMPLES::

            sage: from sage.structure.element_wrapper import DummyParent
            sage: ElementWrapper(DummyParent("A parent"), 1)._ascii_art_()
            1
            sage: x = var('x')
            sage: ElementWrapper(DummyParent("A parent"), x^2 + x)._unicode_art_()
             2
            x  + x
        """
        from sage.typeset.unicode_art import unicode_art
        return unicode_art(self.value)

    def __hash__(self):
        """
        Return the same hash as for the wrapped element.

        EXAMPLES::

            sage: from sage.structure.element_wrapper import DummyParent
            sage: parent1 = DummyParent("A parent")
            sage: parent2 = DummyParent("Another parent")
            sage: hash(ElementWrapper(parent1, 1))
            1
            sage: hash(ElementWrapper(parent2, 1))
            1

        .. TODO::

            Should this take the parent and/or the class into account?
        """
        return hash(self.value)

    def __richcmp__(left, right, int op):
        """
        Return ``True`` if ``left`` compares with ``right`` based on ``op``.

        Default implementation of ``self == other``: two elements are
        equal if they have equal parents and equal values.

        Default implementation of ``self < other``: two elements are
        always incomparable.

        .. NOTE::

            Another option would be to not define ``__lt__``, but
            given the current implementation of SageObject, sorted(...)
            would break.

        TESTS:

        Check that elements of equal-but-not-identical parents compare
        properly (see :trac:`19488`)::

            sage: from sage.misc.nested_class_test import TestParent4
            sage: P = TestParent4()
            sage: Q = TestParent4()
            sage: P == Q
            True
            sage: P is Q
            False
            sage: x = P.an_element(); x
            '_an_element_'
            sage: y = Q.an_element(); y
            '_an_element_'
            sage: x == y
            True
        """
        if isinstance(right, ElementWrapper) and left.parent() == right.parent():
            return left._richcmp_(right, op)
        return coercion_model.richcmp(left, right, op)

    cpdef _richcmp_(left, right, int op):
        """
        Return ``True`` if ``left`` compares with ``right`` based on ``op``.

        TESTS:

        Testing equality::

            sage: from sage.structure.element_wrapper import DummyParent
            sage: parent1 = DummyParent("A parent")
            sage: parent2 = DummyParent("Another parent")
            sage: parent1 == parent2
            False
            sage: l11 = ElementWrapper(parent1, 1)
            sage: l12 = ElementWrapper(parent1, 2)
            sage: l21 = ElementWrapper(parent2, 1)
            sage: l22 = ElementWrapper(parent2, 2)
            sage: l11 == l11
            True
            sage: l11 == l12
            False
            sage: l11 == l21
            False

        Testing inequality::

            sage: from sage.structure.element_wrapper import DummyParent
            sage: parent1 = DummyParent("A parent")
            sage: parent2 = DummyParent("Another parent")
            sage: parent1 == parent2
            False
            sage: l11 = ElementWrapper(parent1, 1)
            sage: l12 = ElementWrapper(parent1, 2)
            sage: l21 = ElementWrapper(parent2, 1)
            sage: l22 = ElementWrapper(parent2, 2)
            sage: l11 != l11
            False
            sage: l11 != l12
            True
            sage: l11 != l21
            True

        Testing less than::

            sage: from sage.structure.element_wrapper import DummyParent
            sage: parent = DummyParent("A parent")
            sage: x = ElementWrapper(parent, 1)
            sage: y = ElementWrapper(parent, 2)
            sage: x.__lt__(x), x.__lt__(y), y.__lt__(x)
            (False, False, False)
            sage: x < x, x < y, y < x
            (False, False, False)
            sage: sorted([x,y])
            [1, 2]
            sage: sorted([y,x])
            [2, 1]
        """
        cdef ElementWrapper self = left
        if op == Py_EQ or op == Py_LE or op == Py_GE:
            return self.value == (<ElementWrapper>right).value
        if op == Py_NE:
            return self.value != (<ElementWrapper>right).value
        return False

    cpdef bint _lt_by_value(self, other):
        """
        Return whether ``self`` is strictly smaller than ``other``.

        With this implementation 'by value', they are always
        incomparable unless ``self`` and ``other`` have the same
        class, parent, and ``self.value < other.value``.

        EXAMPLES::

            sage: from sage.structure.element_wrapper import DummyParent
            sage: class MyElement(ElementWrapper):
            ....:     __lt__ = ElementWrapper._lt_by_value
            ....:
            sage: parent1 = DummyParent("A parent")
            sage: parent2 = DummyParent("Another parent")
            sage: l11 = MyElement(parent1, 1)
            sage: l12 = MyElement(parent1, 2)
            sage: l21 = MyElement(parent2, 1)
            sage: l22 = MyElement(parent2, 2)
            sage: l11 < l11
            False
            sage: l11 < l12, l12 < l11   # values differ
            (True, False)
            sage: l11 < l21              # parents differ
            False
            sage: l11 < 1                # class differ
            False
        """
        return (self.__class__ is other.__class__
                and self._parent is other.parent()
                and self.value < (<ElementWrapper>other).value)

    def __copy__(self):
        """
        Copy ``self`` and in particular its ``value`` attribute.

        EXAMPLES::

            sage: from sage.structure.element_wrapper import DummyParent
            sage: parent = DummyParent("A parent")
            sage: o1 = ElementWrapper(parent, [1]); o1
            [1]
            sage: o2 = copy(o1); o2
            [1]
            sage: o1 is o2, o1.value is o2.value
            (False, False)
            sage: o2.value[0] = 3; o2
            [3]
            sage: o1
            [1]
            sage: class bla(ElementWrapper): pass
            sage: o3 = bla(parent, [1])
            sage: o4 = copy(o3)
            sage: o3.value[0] = 3; o4
            [1]
            sage: o3.__class__
            <class '__main__.bla'>
            sage: o4.__class__
            <class '__main__.bla'>
        """
        # Note : copy(super(ElementWrapper, self)) does not work.
        res = super(ElementWrapper, self).__copy__()
        res.value = copy(self.value)
        return res


class DummyParent(UniqueRepresentation, Parent):
    """
    A class for creating dummy parents for testing :class:`ElementWrapper`
    """
    def __init__(self, name):
        """
        EXAMPLES::

            sage: from sage.structure.element_wrapper import DummyParent
            sage: parent = DummyParent("A Parent")
            sage: skipped = ["_test_an_element", "_test_category",
            ....:            "_test_elements", "_test_elements_eq_reflexive",
            ....:            "_test_elements_eq_symmetric",
            ....:            "_test_elements_eq_transitive",
            ....:            "_test_elements_neq", "_test_some_elements"]
            sage: TestSuite(parent).run(skip=skipped)
        """
        self.name = name

    def _repr_(self):
        """
        EXAMPLES::

            sage: from sage.structure.element_wrapper import DummyParent
            sage: DummyParent("A Parent") # indirect doctest
            A Parent
        """
        return self.name


class ElementWrapperTester(ElementWrapper):
    """
    Test class for the default :meth:`.__copy` method of subclasses of
    :class:`ElementWrapper`.

    TESTS::

        sage: from sage.structure.element_wrapper import ElementWrapperTester
        sage: x = ElementWrapperTester()
        sage: x.append(2); y = copy(x); y.append(42)
        sage: type(y)
        <class 'sage.structure.element_wrapper.ElementWrapperTester'>
        sage: x, y
        ([n=1, value=[2]], [n=2, value=[2, 42]])
        sage: x.append(21); x.append(7)
        sage: x, y
        ([n=3, value=[2, 21, 7]], [n=2, value=[2, 42]])
        sage: x.value, y.value
        ([2, 21, 7], [2, 42])
        sage: x.__dict__, y.__dict__
        ({'n': 3}, {'n': 2})
    """
    def __init__(self):
        """
        TESTS::

            sage: from sage.structure.element_wrapper import ElementWrapperTester
            sage: x = ElementWrapperTester(); x
            [n=0, value=[]]
        """
        from sage.categories.sets_cat import Sets
        super(ElementWrapperTester, self).__init__(Sets().example("facade"), [])
        self.n = 0

    def append(self, x):
        """
        TESTS::

            sage: from sage.structure.element_wrapper import ElementWrapperTester
            sage: x = ElementWrapperTester()
            sage: x.append(2); x
            [n=1, value=[2]]
        """
        self.n +=1
        self.value.append(x)

    def _repr_(self):
        """
        TESTS::

            sage: from sage.structure.element_wrapper import ElementWrapperTester
            sage: x = ElementWrapperTester
            sage: x = ElementWrapperTester(); x
            [n=0, value=[]]
            sage: x.value = [2,32]; x # indirect doctest
            [n=0, value=[2, 32]]
        """
        return "[n=%s, value=%s]" % (self.n, self.value)


cdef class ElementWrapperCheckWrappedClass(ElementWrapper):
    """
    An :class:`element wrapper <ElementWrapper>` such that comparison
    operations are done against subclasses of ``wrapped_class``.
    """
    wrapped_class = object

    def __hash__(self):
        """
        Return the same hash as for the wrapped element.

        EXAMPLES::

            sage: A = cartesian_product([ZZ, ZZ])
            sage: e1 = A((6,9))
            sage: e2 = A((3,8))
            sage: e3 = A((6,9))
            sage: hash(e1) == hash(e2)
            False
            sage: hash(e1) == hash(e3)
            True
        """
        return hash(self.value)

    def __richcmp__(self, right, int op):
        """
        Return ``True`` if ``self`` compares with ``right`` based on ``op``.

        EXAMPLES::

            sage: A = cartesian_product([ZZ, ZZ])
            sage: elt = A((1,1))
            sage: (1, 1) == elt
            True
            sage: elt == (1, 1)
            True
            sage: A((1, 2)) > elt
            True
            sage: elts = [A((x,y)) for y in range(3) for x in range(2)]
            sage: elts
            [(0, 0), (1, 0), (0, 1), (1, 1), (0, 2), (1, 2)]
            sage: sorted(elts)
            [(0, 0), (0, 1), (0, 2), (1, 0), (1, 1), (1, 2)]

        ::

            sage: A = cartesian_product([ZZ, ZZ])
            sage: B = cartesian_product([GF(3), GF(5)])
            sage: A((3,5)) == B((0,0))
            True
        """
        if type(self) is type(right):
            # Both are instances of ElementWrapperCheckWrappedClass:
            # compare using wrapped element if the parents are the same
            other = <ElementWrapperCheckWrappedClass>right
            if self._parent is other._parent:
                return PyObject_RichCompare(self.value, other.value, op)
        elif not isinstance(right, Element):
            # Right is not an Element: compare using wrapped element
            return PyObject_RichCompare(self.value, right, op)
        elif self._parent is (<Element>right)._parent:
            # Different types but same parent? This should not happen
            raise TypeError(f"cannot compare {type(self).__name__} with {type(right).__name__} if parents are equal")

        # Different parents => use coercion model
        return coercion_model.richcmp(self, right, op)



r"""
Examples of finite Weyl groups
"""
#*****************************************************************************
#  Copyright (C) 2008-2009 Nicolas M. Thiery <nthiery at users.sf.net>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#******************************************************************************

from sage.misc.cachefunc import cached_method
from sage.structure.parent import Parent
from sage.structure.element_wrapper import ElementWrapper
from sage.categories.all import FiniteWeylGroups
from sage.structure.unique_representation import UniqueRepresentation

class SymmetricGroup(UniqueRepresentation, Parent):
    r"""
    An example of finite Weyl group: the symmetric group, with
    elements in list notation.

    The purpose of this class is to provide a minimal template for
    implementing finite Weyl groups. See
    :class:`~sage.groups.perm_gps.permgroup_named.SymmetricGroup` for
    a full featured and optimized implementation.

    EXAMPLES::

        sage: S = FiniteWeylGroups().example()
        sage: S
        The symmetric group on {0, ..., 3}
        sage: S.category()
        Category of finite irreducible weyl groups

    The elements of this group are permutations of the set `\{0,\ldots,3\}`::

        sage: S.one()
        (0, 1, 2, 3)
        sage: S.an_element()
        (1, 2, 3, 0)

    The group itself is generated by the elementary transpositions::

        sage: S.simple_reflections()
        Finite family {0: (1, 0, 2, 3), 1: (0, 2, 1, 3), 2: (0, 1, 3, 2)}

    Only the following basic operations are implemented:

    - :meth:`.one`
    - :meth:`.product`
    - :meth:`.simple_reflection`
    - :meth:`.cartan_type`
    - :meth:`.Element.has_right_descent`.

    All the other usual Weyl group operations are inherited from the
    categories::

        sage: S.cardinality()
        24
        sage: S.long_element()
        (3, 2, 1, 0)
        sage: S.cayley_graph(side = "left").plot()
        Graphics object consisting of 120 graphics primitives

    Alternatively, one could have implemented
    :meth:`sage.categories.coxeter_groups.CoxeterGroups.ElementMethods.apply_simple_reflection`
    instead of :meth:`.simple_reflection` and :meth:`.product`. See
    ``CoxeterGroups().example()``.

    TESTS::

        sage: TestSuite(S).run()
    """

    def __init__(self, n = 4):
        """
        EXAMPLES::

            sage: S = sage.categories.examples.finite_weyl_groups.SymmetricGroup(4)
            sage: S == FiniteWeylGroups().example(4)
            True
        """
        Parent.__init__(self, category=FiniteWeylGroups().Irreducible())
        self.n = n

    def _repr_(self):
        """
        EXAMPLES::

            sage: FiniteWeylGroups().example()
            The symmetric group on {0, ..., 3}

        """
        return "The symmetric group on {0, ..., %s}"%(self.n-1)

    @cached_method
    def one(self):
        """
        Implements :meth:`Monoids.ParentMethods.one`.

        EXAMPLES::

            sage: FiniteWeylGroups().example().one()
            (0, 1, 2, 3)
        """
        return self(tuple(range(self.n)))

    def index_set(self):
        """
        Implements :meth:`CoxeterGroups.ParentMethods.index_set`.

        EXAMPLES::

            sage: FiniteWeylGroups().example().index_set()
            [0, 1, 2]
        """
        return list(range(self.n - 1))

    def simple_reflection(self, i):
        """
        Implement :meth:`CoxeterGroups.ParentMethods.simple_reflection`
        by returning the transposition `(i, i+1)`.

        EXAMPLES::

            sage: FiniteWeylGroups().example().simple_reflection(2)
            (0, 1, 3, 2)
        """
        assert i in self.index_set()
        return self(tuple(range(i)) + (i + 1, i) + tuple(range(i + 2, self.n)))

    def cartan_type(self):
        """
        Return the Cartan type of ``self``.

        EXAMPLES::

            sage: FiniteWeylGroups().example().cartan_type()
            ['A', 3] relabelled by {1: 0, 2: 1, 3: 2}
        """
        from sage.combinat.root_system.cartan_type import CartanType
        C = CartanType(['A',self.n-1])
        C = C.relabel(lambda i:i-1)
        return C

    def product(self, x, y):
        """
        Implements :meth:`Semigroups.ParentMethods.product`.

        EXAMPLES::

            sage: s = FiniteWeylGroups().example().simple_reflections()
            sage: s[1] * s[2]
            (0, 2, 3, 1)
            """
        assert x in self
        assert y in self
        return self(tuple(x.value[i] for i in y.value))

    def degrees(self):
        """
        Return the degrees of ``self``.

        EXAMPLES::

            sage: W = FiniteWeylGroups().example()
            sage: W.degrees()
            (2, 3, 4)
        """
        from sage.rings.integer_ring import ZZ
        return tuple(ZZ(i) for i in range(2, self.n + 1))

    class Element(ElementWrapper):

        def has_right_descent(self, i):
            """
            Implements :meth:`CoxeterGroups.ElementMethods.has_right_descent`.

            EXAMPLES::

                sage: S = FiniteWeylGroups().example()
                sage: s = S.simple_reflections()
                sage: (s[1] * s[2]).has_descent(2)
                True
                sage: S._test_has_descent()
            """
            return (self.value[i] > self.value[i+1])


Example = SymmetricGroup



r"""
Kac-Moody Algebras

AUTHORS:

- Travis Scrimshaw (07-15-2017): Initial implementation
"""

# ****************************************************************************
#       Copyright (C) 2017 Travis Scrimshaw <tcscrims at gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************
from sage.misc.cachefunc import cached_method
from sage.categories.category_types import Category_over_base_ring
from sage.categories.lie_algebras import LieAlgebras


class KacMoodyAlgebras(Category_over_base_ring):
    """
    Category of Kac-Moody algebras.
    """
    @cached_method
    def super_categories(self):
        """
        EXAMPLES::

            sage: from sage.categories.kac_moody_algebras import KacMoodyAlgebras
            sage: KacMoodyAlgebras(QQ).super_categories()
            [Category of Lie algebras over Rational Field]
        """
        return [LieAlgebras(self.base_ring())]

    def example(self, n=2):
        """
        Return an example of a Kac-Moody algebra as per
        :meth:`Category.example <sage.categories.category.Category.example>`.

        EXAMPLES::

            sage: from sage.categories.kac_moody_algebras import KacMoodyAlgebras
            sage: KacMoodyAlgebras(QQ).example()
            Lie algebra of ['A', 2] in the Chevalley basis

        We can specify the rank of the example::

            sage: KacMoodyAlgebras(QQ).example(4)
            Lie algebra of ['A', 4] in the Chevalley basis
        """
        from sage.algebras.lie_algebras.classical_lie_algebra import LieAlgebraChevalleyBasis
        return LieAlgebraChevalleyBasis(self.base_ring(), ['A', n])

    class ParentMethods:
        def cartan_type(self):
            """
            Return the Cartan type of ``self``.

            EXAMPLES::

                sage: L = LieAlgebra(QQ, cartan_type=['A', 2])
                sage: L.cartan_type()
                ['A', 2]
            """
            return self._cartan_type

        def weyl_group(self):
            """
            Return the Weyl group of ``self``.

            EXAMPLES::

                sage: L = LieAlgebra(QQ, cartan_type=['A', 2])
                sage: L.weyl_group()
                Weyl Group of type ['A', 2] (as a matrix group acting on the ambient space)
            """
            from sage.combinat.root_system.weyl_group import WeylGroup
            return WeylGroup(self.cartan_type())




from sage.structure.element cimport Element
from .map cimport Map


cdef class Morphism(Map):
    pass

cdef class SetMorphism(Morphism):
    cdef object _function
    cpdef bint _eq_c_impl(left, Element right)



"""
Containers for storing coercion data

This module provides :class:`TripleDict` and :class:`MonoDict`. These are
structures similar to :class:`~weakref.WeakKeyDictionary` in Python's weakref
module, and are optimized for lookup speed. The keys for :class:`TripleDict`
consist of triples (k1,k2,k3) and are looked up by identity rather than
equality. The keys are stored by weakrefs if possible. If any one of the
components k1, k2, k3 gets garbage collected, then the entry is removed from
the :class:`TripleDict`.

Key components that do not allow for weakrefs are stored via a normal
refcounted reference. That means that any entry stored using a triple
(k1,k2,k3) so that none of the k1,k2,k3 allows a weak reference behaves
as an entry in a normal dictionary: Its existence in :class:`TripleDict`
prevents it from being garbage collected.

That container currently is used to store coercion and conversion maps between
two parents (:trac:`715`) and to store homsets of pairs of objects of a
category (:trac:`11521`). In both cases, it is essential that the parent
structures remain garbage collectable, it is essential that the data access is
faster than with a usual :class:`~weakref.WeakKeyDictionary`, and we enforce
the "unique parent condition" in Sage (parent structures should be identical
if they are equal).

:class:`MonoDict` behaves similarly, but it takes a single item as a key. It
is used for caching the parents which allow a coercion map into a fixed other
parent (:trac:`12313`).

By :trac:`14159`, :class:`MonoDict` and :class:`TripleDict` can be optionally
used with weak references on the values.

Note that this kind of dictionary is also used for caching actions and
coerce maps. In previous versions of Sage, the cache was by strong
references and resulted in a memory leak in the following example.
However, this leak was fixed by :trac:`715`, using weak references::

    sage: K.<t> = GF(2^55)
    sage: for i in range(50):
    ....:     a = K.random_element()
    ....:     E = EllipticCurve(j=a)
    ....:     P = E.random_point()
    ....:     Q = 2*P
    sage: L = [Partitions(n) for n in range(200)]  # purge strong cache in CachedRepresentation
    sage: import gc
    sage: n = gc.collect()
    sage: from sage.schemes.elliptic_curves.ell_finite_field import EllipticCurve_finite_field
    sage: LE = [x for x in gc.get_objects() if isinstance(x, EllipticCurve_finite_field)]
    sage: len(LE)
    1
"""

#*****************************************************************************
#       Copyright (C) 2007 Robert Bradshaw <robertwb@math.washington.edu>
#                     2012 Simon King <simon.king@uni-jena.de>
#                     2013 Nils Bruin <nbruin@sfu.ca>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  http://www.gnu.org/licenses/
#*****************************************************************************

cimport cython
from cpython.object cimport *
from cpython.ref cimport Py_XINCREF, Py_XDECREF, Py_CLEAR
from cpython.tuple cimport PyTuple_New
from cpython.weakref cimport PyWeakref_GetObject, PyWeakref_GET_OBJECT
from cysignals.memory cimport check_calloc, sig_free

cdef extern from "Python.h":
    void PyTuple_SET_ITEM(object tuple, Py_ssize_t index, PyObject* item)

cdef extern from "sage/cpython/pyx_visit.h":
    void Py_VISIT3(PyObject*, visitproc, void*)

cdef type KeyedRef, ref
from weakref import KeyedRef, ref

cdef inline bint is_dead_keyedref(x):
    """
    Check whether ``x`` is a ``KeyedRef`` which is dead.
    """
    if type(x) is not KeyedRef:
        return False
    return PyWeakref_GET_OBJECT(x) is <PyObject*>None


# Unique sentinel to indicate a deleted cell
cdef object dummy = object()
cdef PyObject* deleted_key = <PyObject*>dummy


cdef inline bint valid(PyObject* obj):
    """
    Check whether ``obj`` points to a valid object
    """
    return obj is not NULL and obj is not deleted_key


@cython.freelist(256)
cdef class ObjectWrapper:
    """
    A simple fast wrapper around a Python object. This is like a
    1-element tuple except that it does not keep a reference to the
    wrapped object.
    """
    cdef PyObject* obj


cdef inline ObjectWrapper wrap(obj):
    """
    Wrap a given Python object in an :class:`ObjectWrapper`.
    """
    cdef ObjectWrapper w = <ObjectWrapper>(ObjectWrapper.__new__(ObjectWrapper))
    w.obj = <PyObject*>obj
    return w


cdef inline PyObject* unwrap(w) except? NULL:
    """
    Return the object wrapped by an :class:`ObjectWrapper`.
    """
    return (<ObjectWrapper?>w).obj


cdef extract_mono_cell(mono_cell* cell):
    """
    Take the refcounted components from a mono_cell, put them in a
    tuple and return it. The mono_cell itself is marked as "freed".
    The refcounts originally accounting for the presence in the
    mono_cell now account for the presence in the returned tuple,
    which steals those references.

    The returned result is only used to throw away: an advantage is
    that the containing tuple participates in CPython's trashcan,
    which prevents stack overflow on large dereffing cascades.

    A slight disadvantage is that this routine needs to allocate a
    tuple (mainly just to be thrown away)
    """
    assert valid(cell.key_id)
    t = PyTuple_New(2)
    PyTuple_SET_ITEM(t, 0, cell.key_weakref)
    PyTuple_SET_ITEM(t, 1, cell.value)
    cell.key_id = deleted_key
    cell.key_weakref = NULL
    cell.value = NULL
    return t


cdef extract_triple_cell(triple_cell* cell):
    # See extract_mono_cell for documentation
    assert valid(cell.key_id1)
    t = PyTuple_New(4)
    PyTuple_SET_ITEM(t, 0, cell.key_weakref1)
    PyTuple_SET_ITEM(t, 1, cell.key_weakref2)
    PyTuple_SET_ITEM(t, 2, cell.key_weakref3)
    PyTuple_SET_ITEM(t, 3, cell.value)
    cell.key_id1 = deleted_key
    cell.key_id2 = NULL
    cell.key_id3 = NULL
    cell.key_weakref1 = NULL
    cell.key_weakref2 = NULL
    cell.key_weakref3 = NULL
    cell.value = NULL
    return t


cdef class MonoDictEraser:
    """
    Erase items from a :class:`MonoDict` when a weak reference becomes
    invalid.

    This is of internal use only. Instances of this class will be passed as a
    callback function when creating a weak reference.

    EXAMPLES::

        sage: from sage.structure.coerce_dict import MonoDict
        sage: class A: pass
        sage: a = A()
        sage: M = MonoDict()
        sage: M[a] = 1
        sage: len(M)
        1
        sage: del a
        sage: import gc
        sage: n = gc.collect()
        sage: len(M)    # indirect doctest
        0

    AUTHOR:

    - Simon King (2012-01)
    - Nils Bruin (2013-11)
    """
    cdef D

    def __init__(self, D):
        """
        INPUT:

        A :class:`MonoDict`.

        EXAMPLES::

            sage: k = set([1])
            sage: D = sage.structure.coerce_dict.MonoDict([(k,1)])
            sage: len(D)
            1
            sage: del k
            sage: len(D) # indirect doctest
            0
        """
        self.D = ref(D)

    def __call__(self, r):
        """
        INPUT:

        A weak reference with key.

        For internal use only.

        EXAMPLES::

            sage: k = set([1])
            sage: D = sage.structure.coerce_dict.MonoDict([(k,1)])
            sage: len(D)
            1
            sage: del k
            sage: len(D) # indirect doctest
            0
        """
        cdef MonoDict md = <MonoDict>PyWeakref_GetObject(self.D)
        if md is None or not md.mask:
            return
        cdef mono_cell* cursor = md.lookup(unwrap(r.key))
        cdef PyObject* r_ = <PyObject*>r
        if valid(cursor.key_id):
            if cursor.key_weakref is r_ or cursor.value is r_:
                L = extract_mono_cell(cursor)
                md.used -= 1
            else:
                raise AssertionError("MonoDictEraser: key match but no weakref match")


cdef class MonoDict:
    """
    This is a hashtable specifically designed for (read) speed in
    the coercion model.

    It differs from a python WeakKeyDictionary in the following important ways:

       - Comparison is done using the 'is' rather than '==' operator.
       - Only weak references to the keys are stored if at all possible.
         Keys that do not allow for weak references are stored with a normal
         refcounted reference.
       - The callback of the weak references is safe against recursion, see below.

    There are special cdef set/get methods for faster access.
    It is bare-bones in the sense that not all dictionary methods are
    implemented.

    IMPLEMENTATION:

    It is implemented as a hash table with open addressing, similar to python's
    dict.

    INPUT:

    - ``data`` -- optional iterable defining initial data, as dict or
      iterable of (key, value) pairs.

    - ``weak_values`` -- optional bool (default False). If it is true,
      weak references to the values in this dictionary will be used,
      when possible.

    EXAMPLES::

        sage: from sage.structure.coerce_dict import MonoDict
        sage: L = MonoDict()
        sage: a = 'a'; b = 'ab'; c = '-15'
        sage: L[a] = 1
        sage: L[b] = 2
        sage: L[c] = 3

    The key is expected to be a unique object. Hence, the item stored for ``c``
    cannot be obtained by providing another equal string::

        sage: L[a]
        1
        sage: L[b]
        2
        sage: L[c]
        3
        sage: L['-15']
        Traceback (most recent call last):
        ...
        KeyError: '-15'

    Not all features of Python dictionaries are available, but iteration over
    the dictionary items is possible::

        sage: sorted(L.items())
        [('-15', 3), ('a', 1), ('ab', 2)]
        sage: del L[c]
        sage: sorted(L.items())
        [('a', 1), ('ab', 2)]
        sage: len(L)
        2
        sage: for i in range(1000):
        ....:     L[i] = i
        sage: len(L)
        1002
        sage: L['a']
        1
        sage: L['c']
        Traceback (most recent call last):
        ...
        KeyError: 'c'

    TESTS:

    Here, we demonstrate the use of weak values::

        sage: M = MonoDict()
        sage: MW = MonoDict(weak_values=True)
        sage: class Foo: pass
        sage: a = Foo()
        sage: b = Foo()
        sage: k = 1
        sage: M[k] = a
        sage: MW[k] = b
        sage: M[k] is a
        True
        sage: MW[k] is b
        True
        sage: k in M
        True
        sage: k in MW
        True

    While ``M`` uses a strong reference to ``a``, ``MW`` uses a *weak*
    reference to ``b``, and after deleting ``b``, the corresponding item of
    ``MW`` will be removed during the next garbage collection::

        sage: import gc
        sage: del a,b
        sage: _ = gc.collect()
        sage: k in M
        True
        sage: k in MW
        False
        sage: len(MW)
        0
        sage: len(M)
        1

   Note that ``MW`` also accepts values that do not allow for weak references::

        sage: MW[k] = int(5)
        sage: MW[k]
        5

    The following demonstrates that :class:`MonoDict` is safer than
    :class:`~weakref.WeakKeyDictionary` against recursions created by nested
    callbacks; compare :trac:`15069` (the mechanism used now is different, though)::

        sage: M = MonoDict()
        sage: class A: pass
        sage: a = A()
        sage: prev = a
        sage: for i in range(1000):
        ....:     newA = A()
        ....:     M[prev] = newA
        ....:     prev = newA
        sage: len(M)
        1000
        sage: del a
        sage: len(M)
        0

    The corresponding example with a Python :class:`weakref.WeakKeyDictionary`
    would result in a too deep recursion during deletion of the dictionary
    items::

        sage: import weakref
        sage: M = weakref.WeakKeyDictionary()
        sage: a = A()
        sage: prev = a
        sage: for i in range(1000):
        ....:     newA = A()
        ....:     M[prev] = newA
        ....:     prev = newA
        sage: len(M)
        1000

    Check that also in the presence of circular references, :class:`MonoDict`
    gets properly collected::

        sage: import gc
        sage: def count_type(T):
        ....:     return len([c for c in gc.get_objects() if isinstance(c,T)])
        sage: _ = gc.collect()
        sage: N = count_type(MonoDict)
        sage: for i in range(100):
        ....:     V = [MonoDict({"id":j+100*i}) for j in range(100)]
        ....:     n= len(V)
        ....:     for i in range(n): V[i][V[(i+1)%n]]=(i+1)%n
        ....:     del V
        ....:     _ = gc.collect()
        ....:     assert count_type(MonoDict) == N
        sage: count_type(MonoDict) == N
        True

    AUTHORS:

    - Simon King (2012-01)
    - Nils Bruin (2012-08)
    - Simon King (2013-02)
    - Nils Bruin (2013-11)
    """
    cdef mono_cell* lookup(self, PyObject* key):
        """
        Return a pointer to where ``key`` should be stored in this
        :class:`MonoDict`.

        This routine is used for all cases where a (potential) spot for
        a key is looked up. The returned value is a pointer into the dictionary
        store that either contains an entry with the requested key or a free spot
        where an entry for that key should go.
        """
        assert valid(key)

        cdef size_t mask = self.mask
        cdef mono_cell* table = self.table
        cdef mono_cell* first_deleted = NULL

        # Use the memory location of the key as starting point for our
        # hash.
        cdef size_t h = <size_t>key

        # The size of a Python object is at least 2 * sizeof(size_t).
        # Therefore, we don't lose any information by dividing by that.
        # Instead, the lower order bits become more interesting.
        h //= 2 * sizeof(size_t)

        # Bring some higher-order bits in with this permutation.
        cdef size_t i = (h >> 8) ^ h

        cdef size_t perturb = h

        # The probing algorithm is heavily inspired by Python dicts.
        # There is always at least one NULL entry in the store, and the
        # probe sequence eventually covers the entire store (see Theorem
        # below), so the loop below does terminate. Since this loop does
        # not change any refcounts, we know that table will not change
        # during iteration.

        # Theorem: when iterating the function i -> 5*i + 1, every
        # element of Z/(2^n Z) is reached.
        # Proof: define f(x) = 4*x + 1. Then f(5*i + 1) = 5*f(i).
        # Therefore, the iteration is really a transformation of
        # i -> 5*i on the group of 1 mod 4 elements of (Z/2^(n+2) Z).
        # It is a well known fact that this is a cyclic group generated
        # by 5 (or any element which is 5 mod 8).
        cdef mono_cell* cursor
        while True:
            cursor = &(table[i & mask])
            perturb >>= 5
            if cursor.key_id is key:
                return cursor
            elif cursor.key_id is NULL:
                return first_deleted or cursor
            elif cursor.key_id is deleted_key:
                if first_deleted is NULL:
                    first_deleted = cursor
            i = (5*i + 1) + perturb

    cdef int resize(self) except -1:
        """
        Resize dictionary. That can also mean shrink! Size is always a power of 2.
        """
        cdef mono_cell* old_table = self.table
        cdef size_t old_mask = self.mask
        cdef size_t newsize = 8
        cdef size_t minsize = 2 * self.used
        cdef mono_cell* cursor
        cdef mono_cell* entry
        while newsize < minsize:
            newsize *= 2
        cdef mono_cell* table = <mono_cell*>check_calloc(newsize, sizeof(mono_cell))

        # We are done with memory activity. We can move the new (empty)
        # table into place:
        self.table = table
        self.mask = newsize - 1
        self.used = 0
        self.fill = 0

        # We now move all entries over. We are not changing any
        # refcounts here, so this is a very tight loop that doesn't need
        # to worry about tables changing.
        cdef size_t i
        for i in range(old_mask + 1):
            entry = &(old_table[i])
            if valid(entry.key_id):
                cursor = self.lookup(entry.key_id)
                assert cursor.key_id is NULL
                cursor[0] = entry[0]
                self.used += 1
                self.fill += 1
        sig_free(old_table)

    def __cinit__(self):
        """
        Setup basic data structure

        TESTS::

            sage: from sage.structure.coerce_dict import TripleDict
            sage: TripleDict.__new__(TripleDict)
            <sage.structure.coerce_dict.TripleDict object at ...>
        """
        cdef size_t newsize = 8
        # The order is important here: the object must be in a
        # consistent state even if exceptions are raised.
        self.eraser = MonoDictEraser(self)
        self.table = <mono_cell*>check_calloc(newsize, sizeof(mono_cell))
        self.mask = newsize - 1
        self.used = 0
        self.fill = 0

    def __init__(self, data=None, *, weak_values=False):
        """
        Create a special dict using singletons for keys.

        EXAMPLES::

            sage: from sage.structure.coerce_dict import MonoDict
            sage: L = MonoDict()
            sage: a = 'a'
            sage: L[a] = 1
            sage: L[a]
            1
            sage: L = MonoDict({a: 1})
            sage: L[a]
            1
            sage: L = MonoDict([(a, 1)])
            sage: L[a]
            1

        """
        self.weak_values = weak_values
        if data:
            try:
                data = data.items()
            except AttributeError:
                pass
            for k, v in data:
                self.set(k,v)

    def __dealloc__(self):
        MonoDict_clear(self)
        sig_free(self.table)

    def __len__(self):
        """
        The number of items in self.
        EXAMPLES::

            sage: from sage.structure.coerce_dict import MonoDict
            sage: L = MonoDict()
            sage: a = 'a'; b = 'b'; c = 'c'
            sage: L[a] = 1
            sage: L[a] = -1 # re-assign
            sage: L[b] = 1
            sage: L[c] = None
            sage: len(L)
            3
        """
        return self.used

    def __contains__(self, k):
        """
        Test if the dictionary contains a given key.

        EXAMPLES::

            sage: from sage.structure.coerce_dict import MonoDict
            sage: L = MonoDict()
            sage: a = 'a'; b = 'ab'; c = 15
            sage: L[a] = 1
            sage: L[b] = 2
            sage: L[c] = 3
            sage: c in L         # indirect doctest
            True

        The keys are compared by identity, not by equality. Hence, we have::

            sage: c == 15
            True
            sage: 15 in L
            False
        """
        cdef mono_cell* cursor = self.lookup(<PyObject*>k)
        if not valid(cursor.key_id):
            return False
        if not self.weak_values:
            return True
        value = <object>cursor.value
        return not is_dead_keyedref(value)

    def __getitem__(self, k):
        """
        Get the value corresponding to a key.

        EXAMPLES::

            sage: from sage.structure.coerce_dict import MonoDict
            sage: L = MonoDict()
            sage: a = 'a'; b = 'b'; c = 15
            sage: L[a] = 1
            sage: L[b] = 2
            sage: L[c] = 3
            sage: L[c]                  # indirect doctest
            3

        Note that the keys are supposed to be unique::

            sage: c == 15
            True
            sage: c is 15
            False
            sage: L[15]
            Traceback (most recent call last):
            ...
            KeyError: 15
        """
        return self.get(k)

    cdef get(self, k):
        cdef mono_cell* cursor = self.lookup(<PyObject*>k)
        if not valid(cursor.key_id):
            raise KeyError(k)
        # We need to check that the value is a live reference.
        # Items with dead references (in the key or value) are deleted
        # from the MonoDict by the MonoDictEraser. However, if we are
        # in the middle of a deallocation, we may see a dead reference
        # for the value. This cannot happen for the key: we are passed a
        # strong reference to the key as argument of this function, so
        # we know that it's alive.
        value = <object>cursor.value
        if type(value) is KeyedRef:
            value = <object>PyWeakref_GET_OBJECT(value)
            if value is None:
                raise KeyError(k)
        return value

    def __setitem__(self, k, value):
        """
        Set the value corresponding to a key.

        EXAMPLES::

            sage: from sage.structure.coerce_dict import MonoDict
            sage: L = MonoDict()
            sage: a = 'a'
            sage: L[a] = -1   # indirect doctest
            sage: L[a]
            -1
            sage: L[a] = 1
            sage: L[a]
            1
            sage: len(L)
            1
        """
        self.set(k, value)

    cdef int set(self, k, value) except -1:
        cdef mono_cell entry
        cdef PyObject* old_value
        cdef bint maybe_resize = False
        entry.key_id = <PyObject*>k
        if self.weak_values:
            wrap_k = wrap(k)
            try:
                value_store = KeyedRef(value, self.eraser, wrap_k)
                entry.value = <PyObject*>value_store
            except TypeError:
                entry.value = <PyObject*>value
        else:
            entry.value = <PyObject*>value
        Py_XINCREF(entry.value)
        cursor = self.lookup(<PyObject*>k)
        if not valid(cursor.key_id):
            self.used += 1
            if cursor.key_id is NULL:
                self.fill += 1
                maybe_resize = True
            if not self.weak_values:
                wrap_k = wrap(k)
            try:
                key_store = KeyedRef(k, self.eraser, wrap_k)
                entry.key_weakref = <PyObject*>key_store
            except TypeError:
                entry.key_weakref = <PyObject*>k
            Py_XINCREF(entry.key_weakref)

            # We are taking a bit of a gamble here: we're assuming the
            # dictionary has not been resized (otherwise cursor might
            # not be a valid location anymore). The only way in which
            # that could happen is if the allocation activity above
            # forced a GC that triggered code that *adds* entries to
            # this dictionary: the dictionary can only get reshaped if
            # self.fill increases (as happens below). Note that we're
            # holding a strong ref to the dict itself, so that's not
            # liable to disappear. For the truly paranoid: we could
            # detect a change by checking if self.table has changed
            # value.
            cursor[0] = entry

            if maybe_resize and 3*self.fill > 2*self.mask:
                self.resize()
        else:
            old_value = cursor.value
            cursor.value = entry.value
            Py_XDECREF(old_value)

    def __delitem__(self, k):
        """
        Delete the value corresponding to a key.

        EXAMPLES::

            sage: from sage.structure.coerce_dict import MonoDict
            sage: L = MonoDict()
            sage: a = 15
            sage: L[a] = -1
            sage: len(L)
            1

        Note that the keys are unique, hence using a key that is equal but not
        identical to a results in an error::

            sage: del L[15]
            Traceback (most recent call last):
            ...
            KeyError: 15
            sage: a in L
            True
            sage: del L[a]
            sage: len(L)
            0
            sage: a in L
            False
        """
        cdef mono_cell* cursor = self.lookup(<PyObject*>k)
        if not valid(cursor.key_id):
            raise KeyError(k)
        L = extract_mono_cell(cursor)
        self.used -= 1

    def items(self):
        """
        Iterate over the ``(key, value)`` pairs of this :class:`MonoDict`.

        EXAMPLES::

            sage: from sage.structure.coerce_dict import MonoDict
            sage: L = MonoDict()
            sage: L[1] = None
            sage: L[2] = True
            sage: L.items()
            <generator object at ...>
            sage: sorted(L.items())
            [(1, None), (2, True)]
        """
        # Iteration is tricky because the table could change from under
        # us. The following iterates properly if the dictionary does
        # not get resized, which is guaranteed if no NEW entries in the
        # dictionary are introduced. At least we make sure to get our
        # data fresh from "self" every iteration, so that at least we're
        # not reading random memory. If the dictionary changes, it's not
        # guaranteed you get to see any particular entry.
        cdef size_t i = 0
        while i <= self.mask:
            cursor = &(self.table[i])
            i += 1
            if valid(cursor.key_id):
                key = <object>(cursor.key_weakref)
                value = <object>(cursor.value)
                if type(key) is KeyedRef:
                    key = <object>PyWeakref_GET_OBJECT(key)
                    if key is None:
                        print("found defunct key")
                        continue
                if type(value) is KeyedRef:
                    value = <object>PyWeakref_GET_OBJECT(value)
                    if value is None:
                        print("found defunct value")
                        continue
                yield (key, value)

    def copy(self):
        """
        Return a copy of this :class:`MonoDict` as Python dict.

        EXAMPLES::

            sage: from sage.structure.coerce_dict import MonoDict
            sage: L = MonoDict()
            sage: L[1] = 42
            sage: L.copy()
            {1: 42}
        """
        return dict(self.items())

    def __reduce__(self):
        """
        Note that we don't expect equality as this class concerns itself with
        object identity rather than object equality.

        EXAMPLES::

            sage: from sage.structure.coerce_dict import MonoDict
            sage: L = MonoDict()
            sage: L[1] = True
            sage: loads(dumps(L)) == L
            False
            sage: list(loads(dumps(L)).items())
            [(1, True)]
        """
        return MonoDict, (self.copy(),)

# The Cython supplied tp_traverse and tp_clear do not take the
# dynamically allocated table into account, so we have to supply our
# own. The only additional link to follow (that Cython does pick up
# and we have to replicate here) is the "eraser" which in its closure
# stores a reference back to the dictionary itself (meaning that
# MonoDicts only disappear on cyclic GC).
cdef int MonoDict_traverse(MonoDict self, visitproc visit, void* arg):
    if not self.mask:
        return 0
    Py_VISIT3(<PyObject*>self.eraser, visit, arg)
    cdef size_t i
    for i in range(self.mask + 1):
        cursor = &self.table[i]
        if valid(cursor.key_id):
            Py_VISIT3(cursor.key_weakref, visit, arg)
            Py_VISIT3(cursor.value, visit, arg)


cdef int MonoDict_clear(MonoDict self):
    """
    We clear a monodict by taking first taking away the table before
    dereffing its contents. That shortcuts callbacks, so we deref the
    entries straight here. That means this code does not participate in
    Python's trashcan the way that deletion code based on
    extract_mono_cell does, so there is probably a way this code can be
    used to overflow the C stack. It would have to be a pretty devious
    example, though.
    """
    if not self.mask:
        return 0
    cdef size_t mask = self.mask
    self.mask = 0  # Setting mask to 0 immediately prevents recursion
    self.used = 0
    self.fill = 0
    # Set self.eraser to None safely
    cdef object eraser = self.eraser
    self.eraser = None
    for i in range(mask+1):
        cursor = &(self.table[i])
        if valid(cursor.key_id):
            cursor.key_id = deleted_key
            Py_CLEAR(cursor.key_weakref)
            Py_CLEAR(cursor.value)


(<PyTypeObject*>MonoDict).tp_traverse = <traverseproc>(&MonoDict_traverse)
(<PyTypeObject*>MonoDict).tp_clear = <inquiry>(&MonoDict_clear)


cdef class TripleDictEraser:
    """
    Erases items from a :class:`TripleDict` when a weak reference becomes
    invalid.

    This is of internal use only. Instances of this class will be passed as a
    callback function when creating a weak reference.

    EXAMPLES::

        sage: from sage.structure.coerce_dict import TripleDict
        sage: class A: pass
        sage: a = A()
        sage: T = TripleDict()
        sage: T[a,ZZ,None] = 1
        sage: T[ZZ,a,1] = 2
        sage: T[a,a,ZZ] = 3
        sage: len(T)
        3
        sage: del a
        sage: import gc
        sage: n = gc.collect()
        sage: len(T) # indirect doctest
        0

    AUTHOR:

    - Simon King (2012-01)
    - Nils Bruin (2013-11)
    """
    cdef D

    def __init__(self, D):
        """
        INPUT:

        A :class:`TripleDict`. For internal use only.

        EXAMPLES::

            sage: D = sage.structure.coerce_dict.TripleDict()
            sage: k = set([1])
            sage: D[k,1,1] = 1
            sage: len(D)
            1
            sage: del k
            sage: len(D) # indirect doctest
            0

        """
        self.D = ref(D)

    def __call__(self, r):
        """
        INPUT:

        A weak reference with key.

        For internal use only.

        EXAMPLES::

            sage: from sage.structure.coerce_dict import TripleDict
            sage: class A: pass
            sage: a = A()
            sage: T = TripleDict()
            sage: T[a,ZZ,None] = 1
            sage: T[ZZ,a,1] = 2
            sage: T[a,a,ZZ] = 3
            sage: len(T)
            3
            sage: del a
            sage: import gc
            sage: n = gc.collect()
            sage: len(T)    # indirect doctest
            0
        """
        cdef TripleDict td = <TripleDict>PyWeakref_GetObject(self.D)
        if td is None or not td.mask:
            return
        k1, k2, k3 = r.key
        cdef triple_cell* cursor = td.lookup(unwrap(k1), unwrap(k2), unwrap(k3))
        cdef PyObject* r_ = <PyObject*>r
        if valid(cursor.key_id1):
            if (cursor.key_weakref1 is r_ or
                    cursor.key_weakref2 is r_ or
                    cursor.key_weakref3 is r_ or
                    cursor.value is r_):
                L = extract_triple_cell(cursor)
                td.used -= 1
            else:
                raise AssertionError("TripleDictEraser: key match but no weakref match")


cdef class TripleDict:
    """
    This is a hashtable specifically designed for (read) speed in
    the coercion model.

    It differs from a python dict in the following important ways:

       - All keys must be sequence of exactly three elements. All sequence
         types (tuple, list, etc.) map to the same item.

       - Any of the three key components that support weak-refs are stored
         via a weakref. If any of these components gets garbage collected
         then the entire entry is removed. In that sense, this structure
         behaves like a nested :class:`~weakref.WeakKeyDictionary`.

       - Comparison is done using the 'is' rather than '==' operator.

    There are special cdef set/get methods for faster access.
    It is bare-bones in the sense that not all dictionary methods are
    implemented.


    INPUT:

    - ``data`` -- optional iterable defining initial data, as dict or
      iterable of (key, value) pairs.

    - ``weak_values`` -- optional bool (default False). If it is true,
      weak references to the values in this dictionary will be used,
      when possible.


    IMPLEMENTATION:

    It is implemented as a hash table with open addressing, similar to python's
    dict.


    EXAMPLES::

        sage: from sage.structure.coerce_dict import TripleDict
        sage: L = TripleDict()
        sage: a = 'a'; b = 'b'; c = 'c'
        sage: L[a,b,c] = 1
        sage: L[a,b,c]
        1
        sage: L[c,b,a] = -1
        sage: sorted(L.items())
        [(('a', 'b', 'c'), 1), (('c', 'b', 'a'), -1)]
        sage: del L[a,b,c]
        sage: list(L.items())
        [(('c', 'b', 'a'), -1)]
        sage: len(L)
        1
        sage: for i in range(1000):
        ....:     L[i,i,i] = i
        sage: len(L)
        1001
        sage: L = TripleDict(L)
        sage: L[c,b,a]
        -1
        sage: L[a,b,c]
        Traceback (most recent call last):
        ...
        KeyError: ('a', 'b', 'c')
        sage: L[a]
        Traceback (most recent call last):
        ...
        KeyError: 'a'
        sage: L[a] = 1
        Traceback (most recent call last):
        ...
        KeyError: 'a'

    TESTS:

    Here, we demonstrate the use of weak values::

        sage: class Foo: pass
        sage: T = TripleDict()
        sage: TW = TripleDict(weak_values=True)
        sage: a = Foo()
        sage: b = Foo()
        sage: k = 1
        sage: T[a,k,k]=1
        sage: T[k,a,k]=2
        sage: T[k,k,a]=3
        sage: T[k,k,k]=a
        sage: TW[b,k,k]=1
        sage: TW[k,b,k]=2
        sage: TW[k,k,b]=3
        sage: TW[k,k,k]=b
        sage: len(T)
        4
        sage: len(TW)
        4
        sage: (k,k,k) in T
        True
        sage: (k,k,k) in TW
        True
        sage: T[k,k,k] is a
        True
        sage: TW[k,k,k] is b
        True

    Now, ``T`` holds a strong reference to ``a``, namely in ``T[k,k,k]``. Hence,
    when we delete ``a``, *all* items of ``T`` survive::

        sage: import gc
        sage: del a
        sage: _ = gc.collect()
        sage: len(T)
        4

    Only when we remove the strong reference, the items become collectable::

        sage: del T[k,k,k]
        sage: _ = gc.collect()
        sage: len(T)
        0

    The situation is different for ``TW``, since it only holds *weak*
    references to ``a``. Therefore, all items become collectable after
    deleting ``a``::

        sage: del b
        sage: _ = gc.collect()
        sage: len(TW)
        0

    AUTHORS:

    - Robert Bradshaw, 2007-08

    - Simon King, 2012-01

    - Nils Bruin, 2012-08

    - Simon King, 2013-02

    - Nils Bruin, 2013-11
    """
    cdef triple_cell* lookup(self, PyObject* key1, PyObject* key2, PyObject* key3):
        """
        Return a pointer to where ``(key1, key2, key3)`` should be
        stored in this :class:`MonoDict`.

        This routine is used for all cases where a (potential) spot for
        a key is looked up. The returned value is a pointer into the dictionary
        store that either contains an entry with the requested key or a free spot
        where an entry for that key should go.
        """
        cdef size_t mask = self.mask
        cdef triple_cell* table = self.table
        cdef triple_cell* first_deleted = NULL

        # A random linear combination of the memory locations of the keys
        cdef size_t C2 = 0x7de83cbb
        cdef size_t C3 = 0x32354bf3
        cdef size_t h = (<size_t>key1) + C2*(<size_t>key2) + C3*(<size_t>key3)

        # See MonoDict.lookup() for comments about the algorithm
        h //= 2 * sizeof(size_t)

        cdef size_t i = (h >> 8) ^ h
        cdef size_t perturb = h

        cdef triple_cell* cursor
        while True:
            cursor = &(table[i & mask])
            perturb >>= 5
            if cursor.key_id1 is key1:
                if cursor.key_id2 is key2 and cursor.key_id3 is key3:
                    return cursor
            elif cursor.key_id1 is NULL:
                return first_deleted or cursor
            elif cursor.key_id1 is deleted_key:
                if first_deleted is NULL:
                    first_deleted = cursor
            i = (5*i + 1) + perturb

    cdef int resize(self) except -1:
        cdef triple_cell* old_table = self.table
        cdef size_t old_mask = self.mask
        cdef size_t newsize = 8
        cdef size_t minsize = 2 * self.used
        cdef triple_cell* cursor
        cdef triple_cell* entry
        while newsize < minsize:
            newsize *= 2
        cdef triple_cell* table = <triple_cell*>check_calloc(newsize, sizeof(triple_cell))
        self.table = table
        self.mask = newsize - 1
        self.used = 0
        self.fill = 0
        cdef size_t i
        for i in range(old_mask + 1):
            entry = &(old_table[i])
            if valid(entry.key_id1):
                cursor = self.lookup(entry.key_id1, entry.key_id2, entry.key_id3)
                assert cursor.key_id1 is NULL
                cursor[0] = entry[0]
                self.used +=1
                self.fill +=1
        sig_free(old_table)

    def __cinit__(self):
        """
        Setup basic data structure

        TESTS::

            sage: from sage.structure.coerce_dict import MonoDict
            sage: MonoDict.__new__(MonoDict)
            <sage.structure.coerce_dict.MonoDict object at ...>
        """
        cdef size_t newsize = 8
        # The order is important here: the object must be in a
        # consistent state even if exceptions are raised.
        self.eraser = TripleDictEraser(self)
        self.table = <triple_cell*>check_calloc(newsize, sizeof(triple_cell))
        self.mask = newsize - 1
        self.used = 0
        self.fill = 0

    def __init__(self, data=None, *, weak_values=False):
        """
        Create a special dict using triples for keys.

        EXAMPLES::

            sage: from sage.structure.coerce_dict import TripleDict
            sage: L = TripleDict()
            sage: a = 'a'; b = 'b'; c = 'c'
            sage: L[a,b,c] = 1
            sage: L[a,b,c]
            1
            sage: key = ("x", "y", "z")
            sage: L = TripleDict([(key, 42)])
            sage: L[key]
            42
            sage: L = TripleDict({key: 42})
            sage: L[key]
            42

        """
        self.weak_values = weak_values
        if data:
            try:
                data = data.items()
            except AttributeError:
                pass
            for k, v in data:
                self[k] = v

    def __dealloc__(self):
        TripleDict_clear(self)
        sig_free(self.table)

    def __len__(self):
        """
        The number of items in self.

        EXAMPLES::

            sage: from sage.structure.coerce_dict import TripleDict
            sage: L = TripleDict()
            sage: a = 'a'; b = 'b'; c = 'c'
            sage: L[a,b,c] = 1
            sage: L[a,b,c] = -1 # re-assign
            sage: L[a,c,b] = 1
            sage: L[a,a,None] = None
            sage: len(L)
            3
        """
        return self.used

    def __contains__(self, k):
        """
        Test if the dictionary contains a given key.

        EXAMPLES::

            sage: from sage.structure.coerce_dict import TripleDict
            sage: L = TripleDict()
            sage: a = 'a'; b = 'ab'; c = 15
            sage: L[a,b,c] = 123
            sage: (a,b,c) in L         # indirect doctest
            True

        The keys are compared by identity, not by equality. Hence, we have::

            sage: c == 15
            True
            sage: (a, b, 15) in L
            False

        TESTS::

            sage: a in L
            Traceback (most recent call last):
            ...
            KeyError: 'a'
            sage: (a, b) in L
            Traceback (most recent call last):
            ...
            KeyError: ('a', 'ab')
        """
        try:
            k1, k2, k3 = k
        except (TypeError, ValueError):
            raise KeyError(k)
        cdef triple_cell* cursor = self.lookup(<PyObject*>k1, <PyObject*>k2, <PyObject*>k3)
        if not valid(cursor.key_id1):
            return False
        if not self.weak_values:
            return True
        value = <object>cursor.value
        return not is_dead_keyedref(value)

    def __getitem__(self, k):
        """
        Get the value corresponding to a key.

        EXAMPLES::

            sage: from sage.structure.coerce_dict import TripleDict
            sage: L = TripleDict()
            sage: a = 'a'; b = 'b'; c = 'c'
            sage: L[a,b,c] = 1
            sage: L[a,b,c]
            1
        """
        try:
            k1, k2, k3 = k
        except (TypeError, ValueError):
            raise KeyError(k)
        return self.get(k1, k2, k3)

    cdef get(self, k1, k2, k3):
        cdef triple_cell* cursor = self.lookup(<PyObject*>k1, <PyObject*>k2, <PyObject*>k3)
        if not valid(cursor.key_id1):
            raise KeyError((k1, k2, k3))
        value = <object>cursor.value
        if type(value) is KeyedRef:
            value = <object>PyWeakref_GET_OBJECT(value)
            if value is None:
                raise KeyError((k1, k2, k3))
        return value

    def __setitem__(self, k, value):
        """
        Set the value corresponding to a key.

        EXAMPLES::

            sage: from sage.structure.coerce_dict import TripleDict
            sage: L = TripleDict()
            sage: a = 'a'; b = 'b'; c = 'c'
            sage: L[a,b,c] = -1
            sage: L[a,b,c]
            -1
        """
        try:
            k1, k2, k3 = k
        except (TypeError, ValueError):
            raise KeyError(k)
        self.set(k1, k2, k3, value)

    cdef int set(self, k1, k2, k3, value) except -1:
        cdef triple_cell entry
        cdef PyObject* old_value
        cdef bint maybe_resize = False
        entry.key_id1 = <PyObject*>k1
        entry.key_id2 = <PyObject*>k2
        entry.key_id3 = <PyObject*>k3
        if self.weak_values:
            wrap_k = (wrap(k1), wrap(k2), wrap(k3))
            try:
                value_store = KeyedRef(value, self.eraser, wrap_k)
                entry.value = <PyObject*>value_store
            except TypeError:
                entry.value = <PyObject*>value
        else:
            entry.value = <PyObject*>value
        Py_XINCREF(entry.value)
        cursor = self.lookup(<PyObject*>k1, <PyObject*>k2, <PyObject*>k3)
        if not valid(cursor.key_id1):
            self.used += 1
            if cursor.key_id1 is NULL:
                self.fill += 1
                maybe_resize = True
            if not self.weak_values:
                wrap_k = (wrap(k1), wrap(k2), wrap(k3))
            try:
                key_store = KeyedRef(k1, self.eraser, wrap_k)
                entry.key_weakref1 = <PyObject*>key_store
            except TypeError:
                entry.key_weakref1 = <PyObject*>k1
            Py_XINCREF(entry.key_weakref1)
            try:
                key_store = KeyedRef(k2, self.eraser, wrap_k)
                entry.key_weakref2 = <PyObject*>key_store
            except TypeError:
                entry.key_weakref2 = <PyObject*>k2
            Py_XINCREF(entry.key_weakref2)
            try:
                key_store = KeyedRef(k3, self.eraser, wrap_k)
                entry.key_weakref3 = <PyObject*>key_store
            except TypeError:
                entry.key_weakref3 = <PyObject*>k3
            Py_XINCREF(entry.key_weakref3)

            # We are taking a bit of a gamble here: we're assuming the
            # dictionary has not been resized (otherwise cursor might
            # not be a valid location anymore). The only way in which
            # that could happen is if the allocation activity above
            # forced a GC that triggered code that *adds* entries to
            # this dictionary: the dictionary can only get reshaped if
            # self.fill increases (as happens below). Note that we're
            # holding a strong ref to the dict itself, so that's not
            # liable to disappear. For the truly paranoid: we could
            # detect a change by checking if self.table has changed
            # value.
            cursor[0] = entry

            if maybe_resize and 3*self.fill > 2*self.mask:
                self.resize()
        else:
            old_value = cursor.value
            cursor.value = entry.value
            Py_XDECREF(old_value)

    def __delitem__(self, k):
        """
        Delete the value corresponding to a key.

        EXAMPLES::

            sage: from sage.structure.coerce_dict import TripleDict
            sage: L = TripleDict()
            sage: a = 'a'; b = 'b'; c = 'c'
            sage: L[a,b,c] = -1
            sage: (a,b,c) in L
            True
            sage: del L[a,b,c]
            sage: len(L)
            0
            sage: (a,b,c) in L
            False
        """
        try:
            k1, k2, k3 = k
        except (TypeError, ValueError):
            raise KeyError(k)
        cdef triple_cell* cursor = self.lookup(<PyObject*>k1, <PyObject*>k2, <PyObject*>k3)
        if not valid(cursor.key_id1):
            raise KeyError(k)
        L = extract_triple_cell(cursor)
        self.used -= 1

    def items(self):
        """
        Iterate over the ``(key, value)`` pairs of this :class:`TripleDict`.

        EXAMPLES::

            sage: from sage.structure.coerce_dict import TripleDict
            sage: L = TripleDict()
            sage: L[1,2,3] = None
            sage: L.items()
            <generator object at ...>
            sage: list(L.items())
            [((1, 2, 3), None)]
        """
        cdef size_t i = 0
        while i <= self.mask:
            cursor = &(self.table[i])
            i += 1
            if valid(cursor.key_id1):
                key1 = <object>(cursor.key_weakref1)
                key2 = <object>(cursor.key_weakref2)
                key3 = <object>(cursor.key_weakref3)
                value = <object>(cursor.value)
                if type(key1) is KeyedRef:
                    key1 = <object>PyWeakref_GET_OBJECT(key1)
                    if key1 is None:
                        print("found defunct key1")
                        continue
                if type(key2) is KeyedRef:
                    key2 = <object>PyWeakref_GET_OBJECT(key2)
                    if key2 is None:
                        print("found defunct key2")
                        continue
                if type(key3) is KeyedRef:
                    key3 = <object>PyWeakref_GET_OBJECT(key3)
                    if key3 is None:
                        print("found defunct key3")
                        continue
                if type(value) is KeyedRef:
                    value = <object>PyWeakref_GET_OBJECT(value)
                    if value is None:
                        print("found defunct value")
                        continue
                yield ((key1, key2, key3), value)

    def copy(self):
        """
        Return a copy of this :class:`TripleDict` as Python dict.

        EXAMPLES::

            sage: from sage.structure.coerce_dict import TripleDict
            sage: L = TripleDict()
            sage: L[1,2,3] = 42
            sage: L.copy()
            {(1, 2, 3): 42}
        """
        return dict(self.items())

    def __reduce__(self):
        """
        Note that we don't expect equality as this class concerns itself with
        object identity rather than object equality.

        EXAMPLES::

            sage: from sage.structure.coerce_dict import TripleDict
            sage: L = TripleDict()
            sage: L[1,2,3] = True
            sage: loads(dumps(L)) == L
            False
            sage: list(loads(dumps(L)).items())
            [((1, 2, 3), True)]
        """
        return TripleDict, (self.copy(),)

# The Cython supplied tp_traverse and tp_clear do not take the
# dynamically allocated table into account, so we have to supply our
# own. The only additional link to follow (that Cython does pick up
# and we have to replicate here) is the "eraser" which in its closure
# stores a reference back to the dictionary itself (meaning that
# TripleDicts only disappear on cyclic GC).
cdef int TripleDict_traverse(TripleDict self, visitproc visit, void* arg):
    if not self.mask:
        return 0
    Py_VISIT3(<PyObject*>self.eraser, visit, arg)
    cdef size_t i
    for i in range(self.mask + 1):
        cursor = &self.table[i]
        if valid(cursor.key_id1):
            Py_VISIT3(cursor.key_weakref1, visit, arg)
            Py_VISIT3(cursor.key_weakref2, visit, arg)
            Py_VISIT3(cursor.key_weakref3, visit, arg)
            Py_VISIT3(cursor.value, visit, arg)


cdef int TripleDict_clear(TripleDict self):
    if not self.mask:
        return 0
    cdef size_t mask = self.mask
    self.mask = 0  # Setting mask to 0 immediately prevents recursion
    self.used = 0
    self.fill = 0
    # Set self.eraser to None safely
    cdef object eraser = self.eraser
    self.eraser = None
    for i in range(mask + 1):
        cursor = &(self.table[i])
        if valid(cursor.key_id1):
            cursor.key_id1 = deleted_key
            Py_CLEAR(cursor.key_weakref1)
            Py_CLEAR(cursor.key_weakref2)
            Py_CLEAR(cursor.key_weakref3)
            Py_CLEAR(cursor.value)


(<PyTypeObject*>TripleDict).tp_traverse = <traverseproc>(&TripleDict_traverse)
(<PyTypeObject*>TripleDict).tp_clear = <inquiry>(&TripleDict_clear)



r"""
Precision management for non-exact objects

Manage the default precision for non-exact objects such as power series rings or
laurent series rings.

EXAMPLES::

    sage: R.<x> = PowerSeriesRing(QQ)
    sage: R.default_prec()
    20
    sage: cos(x)
    1 - 1/2*x^2 + 1/24*x^4 - 1/720*x^6 + 1/40320*x^8 - 1/3628800*x^10 +
     1/479001600*x^12 - 1/87178291200*x^14 + 1/20922789888000*x^16 -
     1/6402373705728000*x^18 + O(x^20)

::

    sage: R.<x> = PowerSeriesRing(QQ, default_prec=10)
    sage: R.default_prec()
    10
    sage: cos(x)
    1 - 1/2*x^2 + 1/24*x^4 - 1/720*x^6 + 1/40320*x^8 + O(x^10)

.. NOTE::

    Subclasses of :class:`Nonexact` which require to change the default
    precision should implement a method `set_default_prec`.

"""

from sage.rings.integer import Integer

class Nonexact:
    r"""
    A non-exact object with default precision.

    INPUT:

    - ``prec`` -- a non-negative integer representing the default precision of
      ``self`` (default: ``20``)

    """
    def __init__(self, prec=20):
        if prec < 0:
            raise ValueError(f"prec (= {prec}) must be non-negative")
        self._default_prec = Integer(prec)

    def default_prec(self):
        r"""
        Return the default precision for ``self``.

        EXAMPLES::

            sage: R = QQ[[x]]
            sage: R.default_prec()
            20

        ::

            sage: R.<x> = PowerSeriesRing(QQ, default_prec=10)
            sage: R.default_prec()
            10

        """
        try:
            return self._default_prec
        except AttributeError:
            self._default_prec = 20
            return self._default_prec

    def set_default_prec(self, prec):
        r"""
        Set the default precision for ``self``

        .. WARNING::

            This method is outdated. If a subclass of class:`Nonexact` requires
            this method, please overload it instead.

        """
        # TODO: remove in Sage 9.4
        from sage.misc.superseded import deprecation
        msg = "The method set_default_prec() is deprecated and will be removed "
        msg += "in a future version of Sage. The default precision is set "
        msg += "during construction."
        deprecation(18416, msg)
        self._default_prec = Integer(prec)


